# coding: utf-8

"""
    Xero Payroll NZ

    This is the Xero Payroll API for orgs in the NZ region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class Deduction(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "deduction_id": "str",
        "deduction_name": "str",
        "deduction_category": "str",
        "liability_account_id": "str",
        "current_record": "bool",
        "standard_amount": "float",
    }

    attribute_map = {
        "deduction_id": "deductionId",
        "deduction_name": "deductionName",
        "deduction_category": "deductionCategory",
        "liability_account_id": "liabilityAccountId",
        "current_record": "currentRecord",
        "standard_amount": "standardAmount",
    }

    def __init__(
        self,
        deduction_id=None,
        deduction_name=None,
        deduction_category=None,
        liability_account_id=None,
        current_record=None,
        standard_amount=None,
    ):  # noqa: E501
        """Deduction - a model defined in OpenAPI"""  # noqa: E501

        self._deduction_id = None
        self._deduction_name = None
        self._deduction_category = None
        self._liability_account_id = None
        self._current_record = None
        self._standard_amount = None
        self.discriminator = None

        if deduction_id is not None:
            self.deduction_id = deduction_id
        self.deduction_name = deduction_name
        self.deduction_category = deduction_category
        self.liability_account_id = liability_account_id
        if current_record is not None:
            self.current_record = current_record
        if standard_amount is not None:
            self.standard_amount = standard_amount

    @property
    def deduction_id(self):
        """Gets the deduction_id of this Deduction.  # noqa: E501

        The Xero identifier for Deduction  # noqa: E501

        :return: The deduction_id of this Deduction.  # noqa: E501
        :rtype: str
        """
        return self._deduction_id

    @deduction_id.setter
    def deduction_id(self, deduction_id):
        """Sets the deduction_id of this Deduction.

        The Xero identifier for Deduction  # noqa: E501

        :param deduction_id: The deduction_id of this Deduction.  # noqa: E501
        :type: str
        """

        self._deduction_id = deduction_id

    @property
    def deduction_name(self):
        """Gets the deduction_name of this Deduction.  # noqa: E501

        Name of the deduction  # noqa: E501

        :return: The deduction_name of this Deduction.  # noqa: E501
        :rtype: str
        """
        return self._deduction_name

    @deduction_name.setter
    def deduction_name(self, deduction_name):
        """Sets the deduction_name of this Deduction.

        Name of the deduction  # noqa: E501

        :param deduction_name: The deduction_name of this Deduction.  # noqa: E501
        :type: str
        """
        if deduction_name is None:
            raise ValueError(
                "Invalid value for `deduction_name`, must not be `None`"
            )  # noqa: E501

        self._deduction_name = deduction_name

    @property
    def deduction_category(self):
        """Gets the deduction_category of this Deduction.  # noqa: E501

        Deduction Category type  # noqa: E501

        :return: The deduction_category of this Deduction.  # noqa: E501
        :rtype: str
        """
        return self._deduction_category

    @deduction_category.setter
    def deduction_category(self, deduction_category):
        """Sets the deduction_category of this Deduction.

        Deduction Category type  # noqa: E501

        :param deduction_category: The deduction_category of this Deduction.  # noqa: E501
        :type: str
        """
        if deduction_category is None:
            raise ValueError(
                "Invalid value for `deduction_category`, must not be `None`"
            )  # noqa: E501
        allowed_values = [
            "PayrollGiving",
            "KiwiSaverVoluntaryContributions",
            "Superannuation",
            "NzOther",
            "None",
        ]  # noqa: E501

        if deduction_category:
            if deduction_category not in allowed_values:
                raise ValueError(
                    "Invalid value for `deduction_category` ({0}), must be one of {1}".format(  # noqa: E501
                        deduction_category, allowed_values
                    )
                )

        self._deduction_category = deduction_category

    @property
    def liability_account_id(self):
        """Gets the liability_account_id of this Deduction.  # noqa: E501

        Xero identifier for Liability Account  # noqa: E501

        :return: The liability_account_id of this Deduction.  # noqa: E501
        :rtype: str
        """
        return self._liability_account_id

    @liability_account_id.setter
    def liability_account_id(self, liability_account_id):
        """Sets the liability_account_id of this Deduction.

        Xero identifier for Liability Account  # noqa: E501

        :param liability_account_id: The liability_account_id of this Deduction.  # noqa: E501
        :type: str
        """
        if liability_account_id is None:
            raise ValueError(
                "Invalid value for `liability_account_id`, must not be `None`"
            )  # noqa: E501

        self._liability_account_id = liability_account_id

    @property
    def current_record(self):
        """Gets the current_record of this Deduction.  # noqa: E501

        Identifier of a record is active or not.  # noqa: E501

        :return: The current_record of this Deduction.  # noqa: E501
        :rtype: bool
        """
        return self._current_record

    @current_record.setter
    def current_record(self, current_record):
        """Sets the current_record of this Deduction.

        Identifier of a record is active or not.  # noqa: E501

        :param current_record: The current_record of this Deduction.  # noqa: E501
        :type: bool
        """

        self._current_record = current_record

    @property
    def standard_amount(self):
        """Gets the standard_amount of this Deduction.  # noqa: E501

        Standard amount of the deduction.  # noqa: E501

        :return: The standard_amount of this Deduction.  # noqa: E501
        :rtype: float
        """
        return self._standard_amount

    @standard_amount.setter
    def standard_amount(self, standard_amount):
        """Sets the standard_amount of this Deduction.

        Standard amount of the deduction.  # noqa: E501

        :param standard_amount: The standard_amount of this Deduction.  # noqa: E501
        :type: float
        """

        self._standard_amount = standard_amount



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class InvoiceReminder(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"enabled": "bool"}

    attribute_map = {"enabled": "Enabled"}

    def __init__(self, enabled=None):  # noqa: E501
        """InvoiceReminder - a model defined in OpenAPI"""  # noqa: E501

        self._enabled = None
        self.discriminator = None

        if enabled is not None:
            self.enabled = enabled

    @property
    def enabled(self):
        """Gets the enabled of this InvoiceReminder.  # noqa: E501

        setting for on or off  # noqa: E501

        :return: The enabled of this InvoiceReminder.  # noqa: E501
        :rtype: bool
        """
        return self._enabled

    @enabled.setter
    def enabled(self, enabled):
        """Sets the enabled of this InvoiceReminder.

        setting for on or off  # noqa: E501

        :param enabled: The enabled of this InvoiceReminder.  # noqa: E501
        :type: bool
        """

        self._enabled = enabled



# coding: utf-8

"""
    Xero Payroll AU API

    This is the Xero Payroll API for orgs in Australia region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class APIException(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"error_number": "float", "type": "str", "message": "str"}

    attribute_map = {
        "error_number": "ErrorNumber",
        "type": "Type",
        "message": "Message",
    }

    def __init__(self, error_number=None, type=None, message=None):  # noqa: E501
        """APIException - a model defined in OpenAPI"""  # noqa: E501

        self._error_number = None
        self._type = None
        self._message = None
        self.discriminator = None

        if error_number is not None:
            self.error_number = error_number
        if type is not None:
            self.type = type
        if message is not None:
            self.message = message

    @property
    def error_number(self):
        """Gets the error_number of this APIException.  # noqa: E501

        The error number  # noqa: E501

        :return: The error_number of this APIException.  # noqa: E501
        :rtype: float
        """
        return self._error_number

    @error_number.setter
    def error_number(self, error_number):
        """Sets the error_number of this APIException.

        The error number  # noqa: E501

        :param error_number: The error_number of this APIException.  # noqa: E501
        :type: float
        """

        self._error_number = error_number

    @property
    def type(self):
        """Gets the type of this APIException.  # noqa: E501

        The type of error  # noqa: E501

        :return: The type of this APIException.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this APIException.

        The type of error  # noqa: E501

        :param type: The type of this APIException.  # noqa: E501
        :type: str
        """

        self._type = type

    @property
    def message(self):
        """Gets the message of this APIException.  # noqa: E501

        The message describing the error  # noqa: E501

        :return: The message of this APIException.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this APIException.

        The message describing the error  # noqa: E501

        :param message: The message of this APIException.  # noqa: E501
        :type: str
        """

        self._message = message



# -*- coding: utf-8 -*-

from xero_python import exceptions


def translate_status_exception(error, api_instance, api_method):
    """
    Translate HTTPStatusException into proper Xero API status error
    :param error: HTTPStatusException
    :param api_instance: xero api instance exception occurred to
    :param api_method: xero api instance method name exception occurred to
    :return: HTTPStatusException
    """
    if error.status == 400:
        return exceptions.AccountingBadRequestException(http_resp=error.http_resp)
    return exceptions.translate_status_exception(error, api_instance, api_method)



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class PaymentServices(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"payment_services": "list[PaymentService]"}

    attribute_map = {"payment_services": "PaymentServices"}

    def __init__(self, payment_services=None):  # noqa: E501
        """PaymentServices - a model defined in OpenAPI"""  # noqa: E501

        self._payment_services = None
        self.discriminator = None

        if payment_services is not None:
            self.payment_services = payment_services

    @property
    def payment_services(self):
        """Gets the payment_services of this PaymentServices.  # noqa: E501


        :return: The payment_services of this PaymentServices.  # noqa: E501
        :rtype: list[PaymentService]
        """
        return self._payment_services

    @payment_services.setter
    def payment_services(self, payment_services):
        """Sets the payment_services of this PaymentServices.


        :param payment_services: The payment_services of this PaymentServices.  # noqa: E501
        :type: list[PaymentService]
        """

        self._payment_services = payment_services



# coding: utf-8

"""
    Xero Payroll UK

    This is the Xero Payroll API for orgs in the UK region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class Deduction(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "deduction_id": "str",
        "deduction_name": "str",
        "deduction_category": "str",
        "liability_account_id": "str",
        "current_record": "bool",
        "standard_amount": "float",
        "reduces_super_liability": "bool",
        "reduces_tax_liability": "bool",
        "calculation_type": "str",
        "percentage": "float",
        "subject_to_nic": "bool",
        "subject_to_tax": "bool",
        "is_reduced_by_basic_rate": "bool",
        "apply_to_pension_calculations": "bool",
        "is_calculating_on_qualifying_earnings": "bool",
        "is_pension": "bool",
    }

    attribute_map = {
        "deduction_id": "deductionId",
        "deduction_name": "deductionName",
        "deduction_category": "deductionCategory",
        "liability_account_id": "liabilityAccountId",
        "current_record": "currentRecord",
        "standard_amount": "standardAmount",
        "reduces_super_liability": "reducesSuperLiability",
        "reduces_tax_liability": "reducesTaxLiability",
        "calculation_type": "calculationType",
        "percentage": "percentage",
        "subject_to_nic": "subjectToNIC",
        "subject_to_tax": "subjectToTax",
        "is_reduced_by_basic_rate": "isReducedByBasicRate",
        "apply_to_pension_calculations": "applyToPensionCalculations",
        "is_calculating_on_qualifying_earnings": "isCalculatingOnQualifyingEarnings",
        "is_pension": "isPension",
    }

    def __init__(
        self,
        deduction_id=None,
        deduction_name=None,
        deduction_category=None,
        liability_account_id=None,
        current_record=None,
        standard_amount=None,
        reduces_super_liability=None,
        reduces_tax_liability=None,
        calculation_type=None,
        percentage=None,
        subject_to_nic=None,
        subject_to_tax=None,
        is_reduced_by_basic_rate=None,
        apply_to_pension_calculations=None,
        is_calculating_on_qualifying_earnings=None,
        is_pension=None,
    ):  # noqa: E501
        """Deduction - a model defined in OpenAPI"""  # noqa: E501

        self._deduction_id = None
        self._deduction_name = None
        self._deduction_category = None
        self._liability_account_id = None
        self._current_record = None
        self._standard_amount = None
        self._reduces_super_liability = None
        self._reduces_tax_liability = None
        self._calculation_type = None
        self._percentage = None
        self._subject_to_nic = None
        self._subject_to_tax = None
        self._is_reduced_by_basic_rate = None
        self._apply_to_pension_calculations = None
        self._is_calculating_on_qualifying_earnings = None
        self._is_pension = None
        self.discriminator = None

        if deduction_id is not None:
            self.deduction_id = deduction_id
        self.deduction_name = deduction_name
        if deduction_category is not None:
            self.deduction_category = deduction_category
        self.liability_account_id = liability_account_id
        if current_record is not None:
            self.current_record = current_record
        if standard_amount is not None:
            self.standard_amount = standard_amount
        if reduces_super_liability is not None:
            self.reduces_super_liability = reduces_super_liability
        if reduces_tax_liability is not None:
            self.reduces_tax_liability = reduces_tax_liability
        if calculation_type is not None:
            self.calculation_type = calculation_type
        if percentage is not None:
            self.percentage = percentage
        if subject_to_nic is not None:
            self.subject_to_nic = subject_to_nic
        if subject_to_tax is not None:
            self.subject_to_tax = subject_to_tax
        if is_reduced_by_basic_rate is not None:
            self.is_reduced_by_basic_rate = is_reduced_by_basic_rate
        if apply_to_pension_calculations is not None:
            self.apply_to_pension_calculations = apply_to_pension_calculations
        if is_calculating_on_qualifying_earnings is not None:
            self.is_calculating_on_qualifying_earnings = (
                is_calculating_on_qualifying_earnings
            )
        if is_pension is not None:
            self.is_pension = is_pension

    @property
    def deduction_id(self):
        """Gets the deduction_id of this Deduction.  # noqa: E501

        The Xero identifier for Deduction  # noqa: E501

        :return: The deduction_id of this Deduction.  # noqa: E501
        :rtype: str
        """
        return self._deduction_id

    @deduction_id.setter
    def deduction_id(self, deduction_id):
        """Sets the deduction_id of this Deduction.

        The Xero identifier for Deduction  # noqa: E501

        :param deduction_id: The deduction_id of this Deduction.  # noqa: E501
        :type: str
        """

        self._deduction_id = deduction_id

    @property
    def deduction_name(self):
        """Gets the deduction_name of this Deduction.  # noqa: E501

        Name of the deduction  # noqa: E501

        :return: The deduction_name of this Deduction.  # noqa: E501
        :rtype: str
        """
        return self._deduction_name

    @deduction_name.setter
    def deduction_name(self, deduction_name):
        """Sets the deduction_name of this Deduction.

        Name of the deduction  # noqa: E501

        :param deduction_name: The deduction_name of this Deduction.  # noqa: E501
        :type: str
        """
        if deduction_name is None:
            raise ValueError(
                "Invalid value for `deduction_name`, must not be `None`"
            )  # noqa: E501

        self._deduction_name = deduction_name

    @property
    def deduction_category(self):
        """Gets the deduction_category of this Deduction.  # noqa: E501

        Deduction Category type  # noqa: E501

        :return: The deduction_category of this Deduction.  # noqa: E501
        :rtype: str
        """
        return self._deduction_category

    @deduction_category.setter
    def deduction_category(self, deduction_category):
        """Sets the deduction_category of this Deduction.

        Deduction Category type  # noqa: E501

        :param deduction_category: The deduction_category of this Deduction.  # noqa: E501
        :type: str
        """
        allowed_values = [
            "CapitalContributions",
            "ChildCareVoucher",
            "MakingGood",
            "PostgraduateLoanDeductions",
            "PrivateUsePayments",
            "SalarySacrifice",
            "StakeholderPension",
            "StakeholderPensionPostTax",
            "StudentLoanDeductions",
            "UkOther",
            "None",
        ]  # noqa: E501

        if deduction_category:
            if deduction_category not in allowed_values:
                raise ValueError(
                    "Invalid value for `deduction_category` ({0}), must be one of {1}".format(  # noqa: E501
                        deduction_category, allowed_values
                    )
                )

        self._deduction_category = deduction_category

    @property
    def liability_account_id(self):
        """Gets the liability_account_id of this Deduction.  # noqa: E501

        Xero identifier for Liability Account  # noqa: E501

        :return: The liability_account_id of this Deduction.  # noqa: E501
        :rtype: str
        """
        return self._liability_account_id

    @liability_account_id.setter
    def liability_account_id(self, liability_account_id):
        """Sets the liability_account_id of this Deduction.

        Xero identifier for Liability Account  # noqa: E501

        :param liability_account_id: The liability_account_id of this Deduction.  # noqa: E501
        :type: str
        """
        if liability_account_id is None:
            raise ValueError(
                "Invalid value for `liability_account_id`, must not be `None`"
            )  # noqa: E501

        self._liability_account_id = liability_account_id

    @property
    def current_record(self):
        """Gets the current_record of this Deduction.  # noqa: E501

        Identifier of a record is active or not.  # noqa: E501

        :return: The current_record of this Deduction.  # noqa: E501
        :rtype: bool
        """
        return self._current_record

    @current_record.setter
    def current_record(self, current_record):
        """Sets the current_record of this Deduction.

        Identifier of a record is active or not.  # noqa: E501

        :param current_record: The current_record of this Deduction.  # noqa: E501
        :type: bool
        """

        self._current_record = current_record

    @property
    def standard_amount(self):
        """Gets the standard_amount of this Deduction.  # noqa: E501

        Standard amount of the deduction  # noqa: E501

        :return: The standard_amount of this Deduction.  # noqa: E501
        :rtype: float
        """
        return self._standard_amount

    @standard_amount.setter
    def standard_amount(self, standard_amount):
        """Sets the standard_amount of this Deduction.

        Standard amount of the deduction  # noqa: E501

        :param standard_amount: The standard_amount of this Deduction.  # noqa: E501
        :type: float
        """

        self._standard_amount = standard_amount

    @property
    def reduces_super_liability(self):
        """Gets the reduces_super_liability of this Deduction.  # noqa: E501

        Identifier of reduces super liability  # noqa: E501

        :return: The reduces_super_liability of this Deduction.  # noqa: E501
        :rtype: bool
        """
        return self._reduces_super_liability

    @reduces_super_liability.setter
    def reduces_super_liability(self, reduces_super_liability):
        """Sets the reduces_super_liability of this Deduction.

        Identifier of reduces super liability  # noqa: E501

        :param reduces_super_liability: The reduces_super_liability of this Deduction.  # noqa: E501
        :type: bool
        """

        self._reduces_super_liability = reduces_super_liability

    @property
    def reduces_tax_liability(self):
        """Gets the reduces_tax_liability of this Deduction.  # noqa: E501

        Identifier of reduces tax liability  # noqa: E501

        :return: The reduces_tax_liability of this Deduction.  # noqa: E501
        :rtype: bool
        """
        return self._reduces_tax_liability

    @reduces_tax_liability.setter
    def reduces_tax_liability(self, reduces_tax_liability):
        """Sets the reduces_tax_liability of this Deduction.

        Identifier of reduces tax liability  # noqa: E501

        :param reduces_tax_liability: The reduces_tax_liability of this Deduction.  # noqa: E501
        :type: bool
        """

        self._reduces_tax_liability = reduces_tax_liability

    @property
    def calculation_type(self):
        """Gets the calculation_type of this Deduction.  # noqa: E501

        determine the calculation type whether fixed amount or percentage of gross  # noqa: E501

        :return: The calculation_type of this Deduction.  # noqa: E501
        :rtype: str
        """
        return self._calculation_type

    @calculation_type.setter
    def calculation_type(self, calculation_type):
        """Sets the calculation_type of this Deduction.

        determine the calculation type whether fixed amount or percentage of gross  # noqa: E501

        :param calculation_type: The calculation_type of this Deduction.  # noqa: E501
        :type: str
        """
        allowed_values = ["FixedAmount", "PercentageOfGross", "None"]  # noqa: E501

        if calculation_type:
            if calculation_type not in allowed_values:
                raise ValueError(
                    "Invalid value for `calculation_type` ({0}), must be one of {1}".format(  # noqa: E501
                        calculation_type, allowed_values
                    )
                )

        self._calculation_type = calculation_type

    @property
    def percentage(self):
        """Gets the percentage of this Deduction.  # noqa: E501

        Percentage of gross  # noqa: E501

        :return: The percentage of this Deduction.  # noqa: E501
        :rtype: float
        """
        return self._percentage

    @percentage.setter
    def percentage(self, percentage):
        """Sets the percentage of this Deduction.

        Percentage of gross  # noqa: E501

        :param percentage: The percentage of this Deduction.  # noqa: E501
        :type: float
        """

        self._percentage = percentage

    @property
    def subject_to_nic(self):
        """Gets the subject_to_nic of this Deduction.  # noqa: E501

        Identifier of subject To NIC  # noqa: E501

        :return: The subject_to_nic of this Deduction.  # noqa: E501
        :rtype: bool
        """
        return self._subject_to_nic

    @subject_to_nic.setter
    def subject_to_nic(self, subject_to_nic):
        """Sets the subject_to_nic of this Deduction.

        Identifier of subject To NIC  # noqa: E501

        :param subject_to_nic: The subject_to_nic of this Deduction.  # noqa: E501
        :type: bool
        """

        self._subject_to_nic = subject_to_nic

    @property
    def subject_to_tax(self):
        """Gets the subject_to_tax of this Deduction.  # noqa: E501

        Identifier of subject To Tax  # noqa: E501

        :return: The subject_to_tax of this Deduction.  # noqa: E501
        :rtype: bool
        """
        return self._subject_to_tax

    @subject_to_tax.setter
    def subject_to_tax(self, subject_to_tax):
        """Sets the subject_to_tax of this Deduction.

        Identifier of subject To Tax  # noqa: E501

        :param subject_to_tax: The subject_to_tax of this Deduction.  # noqa: E501
        :type: bool
        """

        self._subject_to_tax = subject_to_tax

    @property
    def is_reduced_by_basic_rate(self):
        """Gets the is_reduced_by_basic_rate of this Deduction.  # noqa: E501

        Identifier of reduced by basic rate applicable or not  # noqa: E501

        :return: The is_reduced_by_basic_rate of this Deduction.  # noqa: E501
        :rtype: bool
        """
        return self._is_reduced_by_basic_rate

    @is_reduced_by_basic_rate.setter
    def is_reduced_by_basic_rate(self, is_reduced_by_basic_rate):
        """Sets the is_reduced_by_basic_rate of this Deduction.

        Identifier of reduced by basic rate applicable or not  # noqa: E501

        :param is_reduced_by_basic_rate: The is_reduced_by_basic_rate of this Deduction.  # noqa: E501
        :type: bool
        """

        self._is_reduced_by_basic_rate = is_reduced_by_basic_rate

    @property
    def apply_to_pension_calculations(self):
        """Gets the apply_to_pension_calculations of this Deduction.  # noqa: E501

        Identifier for apply to pension calculations  # noqa: E501

        :return: The apply_to_pension_calculations of this Deduction.  # noqa: E501
        :rtype: bool
        """
        return self._apply_to_pension_calculations

    @apply_to_pension_calculations.setter
    def apply_to_pension_calculations(self, apply_to_pension_calculations):
        """Sets the apply_to_pension_calculations of this Deduction.

        Identifier for apply to pension calculations  # noqa: E501

        :param apply_to_pension_calculations: The apply_to_pension_calculations of this Deduction.  # noqa: E501
        :type: bool
        """

        self._apply_to_pension_calculations = apply_to_pension_calculations

    @property
    def is_calculating_on_qualifying_earnings(self):
        """Gets the is_calculating_on_qualifying_earnings of this Deduction.  # noqa: E501

        Identifier of calculating on qualifying earnings  # noqa: E501

        :return: The is_calculating_on_qualifying_earnings of this Deduction.  # noqa: E501
        :rtype: bool
        """
        return self._is_calculating_on_qualifying_earnings

    @is_calculating_on_qualifying_earnings.setter
    def is_calculating_on_qualifying_earnings(
        self, is_calculating_on_qualifying_earnings
    ):
        """Sets the is_calculating_on_qualifying_earnings of this Deduction.

        Identifier of calculating on qualifying earnings  # noqa: E501

        :param is_calculating_on_qualifying_earnings: The is_calculating_on_qualifying_earnings of this Deduction.  # noqa: E501
        :type: bool
        """

        self._is_calculating_on_qualifying_earnings = (
            is_calculating_on_qualifying_earnings
        )

    @property
    def is_pension(self):
        """Gets the is_pension of this Deduction.  # noqa: E501

        Identifier of applicable for pension or not  # noqa: E501

        :return: The is_pension of this Deduction.  # noqa: E501
        :rtype: bool
        """
        return self._is_pension

    @is_pension.setter
    def is_pension(self, is_pension):
        """Sets the is_pension of this Deduction.

        Identifier of applicable for pension or not  # noqa: E501

        :param is_pension: The is_pension of this Deduction.  # noqa: E501
        :type: bool
        """

        self._is_pension = is_pension



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class ImportSummary(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "accounts": "ImportSummaryAccounts",
        "organisation": "ImportSummaryOrganisation",
    }

    attribute_map = {"accounts": "Accounts", "organisation": "Organisation"}

    def __init__(self, accounts=None, organisation=None):  # noqa: E501
        """ImportSummary - a model defined in OpenAPI"""  # noqa: E501

        self._accounts = None
        self._organisation = None
        self.discriminator = None

        if accounts is not None:
            self.accounts = accounts
        if organisation is not None:
            self.organisation = organisation

    @property
    def accounts(self):
        """Gets the accounts of this ImportSummary.  # noqa: E501


        :return: The accounts of this ImportSummary.  # noqa: E501
        :rtype: ImportSummaryAccounts
        """
        return self._accounts

    @accounts.setter
    def accounts(self, accounts):
        """Sets the accounts of this ImportSummary.


        :param accounts: The accounts of this ImportSummary.  # noqa: E501
        :type: ImportSummaryAccounts
        """

        self._accounts = accounts

    @property
    def organisation(self):
        """Gets the organisation of this ImportSummary.  # noqa: E501


        :return: The organisation of this ImportSummary.  # noqa: E501
        :rtype: ImportSummaryOrganisation
        """
        return self._organisation

    @organisation.setter
    def organisation(self, organisation):
        """Sets the organisation of this ImportSummary.


        :param organisation: The organisation of this ImportSummary.  # noqa: E501
        :type: ImportSummaryOrganisation
        """

        self._organisation = organisation



# coding: utf-8

"""
    Xero Payroll UK

    This is the Xero Payroll API for orgs in the UK region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class DeductionObject(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "pagination": "Pagination",
        "problem": "Problem",
        "deduction": "Deduction",
    }

    attribute_map = {
        "pagination": "pagination",
        "problem": "problem",
        "deduction": "deduction",
    }

    def __init__(self, pagination=None, problem=None, deduction=None):  # noqa: E501
        """DeductionObject - a model defined in OpenAPI"""  # noqa: E501

        self._pagination = None
        self._problem = None
        self._deduction = None
        self.discriminator = None

        if pagination is not None:
            self.pagination = pagination
        if problem is not None:
            self.problem = problem
        if deduction is not None:
            self.deduction = deduction

    @property
    def pagination(self):
        """Gets the pagination of this DeductionObject.  # noqa: E501


        :return: The pagination of this DeductionObject.  # noqa: E501
        :rtype: Pagination
        """
        return self._pagination

    @pagination.setter
    def pagination(self, pagination):
        """Sets the pagination of this DeductionObject.


        :param pagination: The pagination of this DeductionObject.  # noqa: E501
        :type: Pagination
        """

        self._pagination = pagination

    @property
    def problem(self):
        """Gets the problem of this DeductionObject.  # noqa: E501


        :return: The problem of this DeductionObject.  # noqa: E501
        :rtype: Problem
        """
        return self._problem

    @problem.setter
    def problem(self, problem):
        """Sets the problem of this DeductionObject.


        :param problem: The problem of this DeductionObject.  # noqa: E501
        :type: Problem
        """

        self._problem = problem

    @property
    def deduction(self):
        """Gets the deduction of this DeductionObject.  # noqa: E501


        :return: The deduction of this DeductionObject.  # noqa: E501
        :rtype: Deduction
        """
        return self._deduction

    @deduction.setter
    def deduction(self, deduction):
        """Sets the deduction of this DeductionObject.


        :param deduction: The deduction of this DeductionObject.  # noqa: E501
        :type: Deduction
        """

        self._deduction = deduction



# -*- coding: utf-8 -*-
from functools import singledispatch


def getvalue(value, path, default=None):
    """Get a named attribute or item from an object; getvalue(x, 'y.z')
       is equivalent to x.y.z
    When a default argument is given, it is returned when the attribute doesn't
    exist; without it, None returned
    List indexing supported x.0.y.1.z etc.
    """
    paths = path.split(".")
    return get_nested(value, *paths, default=default)


@singledispatch
def get_nested(value, path, *paths, default=None):
    value = getattr(value, path, default)
    if paths:
        return get_nested(value, *paths, default=default)

    return value


@get_nested.register(dict)
def get_nested_dict(value, path, *paths, default=None):
    value = value.get(path, default)
    if paths:
        return get_nested(value, *paths, default=default)

    return value


@get_nested.register(list)
@get_nested.register(tuple)
def get_nested_list(value, path, *paths, default=None):
    try:
        value = value[int(path)]
    except IndexError:
        value = default

    if paths:
        return get_nested(value, *paths, default=default)

    return value



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from enum import Enum


class AccountType(Enum):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    allowed enum values
    """

    BANK = "BANK"
    CURRENT = "CURRENT"
    CURRLIAB = "CURRLIAB"
    DEPRECIATN = "DEPRECIATN"
    DIRECTCOSTS = "DIRECTCOSTS"
    EQUITY = "EQUITY"
    EXPENSE = "EXPENSE"
    FIXED = "FIXED"
    INVENTORY = "INVENTORY"
    LIABILITY = "LIABILITY"
    NONCURRENT = "NONCURRENT"
    OTHERINCOME = "OTHERINCOME"
    OVERHEADS = "OVERHEADS"
    PREPAYMENT = "PREPAYMENT"
    REVENUE = "REVENUE"
    SALES = "SALES"
    TERMLIAB = "TERMLIAB"
    PAYGLIABILITY = "PAYGLIABILITY"
    PAYG = "PAYG"
    SUPERANNUATIONEXPENSE = "SUPERANNUATIONEXPENSE"
    SUPERANNUATIONLIABILITY = "SUPERANNUATIONLIABILITY"
    WAGESEXPENSE = "WAGESEXPENSE"



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class ContactGroup(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "name": "str",
        "status": "str",
        "contact_group_id": "str",
        "contacts": "list[Contact]",
    }

    attribute_map = {
        "name": "Name",
        "status": "Status",
        "contact_group_id": "ContactGroupID",
        "contacts": "Contacts",
    }

    def __init__(
        self, name=None, status=None, contact_group_id=None, contacts=None
    ):  # noqa: E501
        """ContactGroup - a model defined in OpenAPI"""  # noqa: E501

        self._name = None
        self._status = None
        self._contact_group_id = None
        self._contacts = None
        self.discriminator = None

        if name is not None:
            self.name = name
        if status is not None:
            self.status = status
        if contact_group_id is not None:
            self.contact_group_id = contact_group_id
        if contacts is not None:
            self.contacts = contacts

    @property
    def name(self):
        """Gets the name of this ContactGroup.  # noqa: E501

        The Name of the contact group. Required when creating a new contact  group  # noqa: E501

        :return: The name of this ContactGroup.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ContactGroup.

        The Name of the contact group. Required when creating a new contact  group  # noqa: E501

        :param name: The name of this ContactGroup.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def status(self):
        """Gets the status of this ContactGroup.  # noqa: E501

        The Status of a contact group. To delete a contact group update the status to DELETED. Only contact groups with a status of ACTIVE are returned on GETs.  # noqa: E501

        :return: The status of this ContactGroup.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this ContactGroup.

        The Status of a contact group. To delete a contact group update the status to DELETED. Only contact groups with a status of ACTIVE are returned on GETs.  # noqa: E501

        :param status: The status of this ContactGroup.  # noqa: E501
        :type: str
        """
        allowed_values = ["ACTIVE", "DELETED", "None"]  # noqa: E501

        if status:
            if status not in allowed_values:
                raise ValueError(
                    "Invalid value for `status` ({0}), must be one of {1}".format(  # noqa: E501
                        status, allowed_values
                    )
                )

        self._status = status

    @property
    def contact_group_id(self):
        """Gets the contact_group_id of this ContactGroup.  # noqa: E501

        The Xero identifier for an contact group – specified as a string following the endpoint name. e.g. /297c2dc5-cc47-4afd-8ec8-74990b8761e9  # noqa: E501

        :return: The contact_group_id of this ContactGroup.  # noqa: E501
        :rtype: str
        """
        return self._contact_group_id

    @contact_group_id.setter
    def contact_group_id(self, contact_group_id):
        """Sets the contact_group_id of this ContactGroup.

        The Xero identifier for an contact group – specified as a string following the endpoint name. e.g. /297c2dc5-cc47-4afd-8ec8-74990b8761e9  # noqa: E501

        :param contact_group_id: The contact_group_id of this ContactGroup.  # noqa: E501
        :type: str
        """

        self._contact_group_id = contact_group_id

    @property
    def contacts(self):
        """Gets the contacts of this ContactGroup.  # noqa: E501

        The ContactID and Name of Contacts in a contact group. Returned on GETs when the ContactGroupID is supplied in the URL.  # noqa: E501

        :return: The contacts of this ContactGroup.  # noqa: E501
        :rtype: list[Contact]
        """
        return self._contacts

    @contacts.setter
    def contacts(self, contacts):
        """Sets the contacts of this ContactGroup.

        The ContactID and Name of Contacts in a contact group. Returned on GETs when the ContactGroupID is supplied in the URL.  # noqa: E501

        :param contacts: The contacts of this ContactGroup.  # noqa: E501
        :type: list[Contact]
        """

        self._contacts = contacts



# coding: utf-8

"""
    Xero Payroll NZ

    This is the Xero Payroll API for orgs in the NZ region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class GrossEarningsHistory(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"days_paid": "int", "unpaid_weeks": "int"}

    attribute_map = {"days_paid": "daysPaid", "unpaid_weeks": "unpaidWeeks"}

    def __init__(self, days_paid=None, unpaid_weeks=None):  # noqa: E501
        """GrossEarningsHistory - a model defined in OpenAPI"""  # noqa: E501

        self._days_paid = None
        self._unpaid_weeks = None
        self.discriminator = None

        if days_paid is not None:
            self.days_paid = days_paid
        if unpaid_weeks is not None:
            self.unpaid_weeks = unpaid_weeks

    @property
    def days_paid(self):
        """Gets the days_paid of this GrossEarningsHistory.  # noqa: E501

        Number of days the employee worked in the pay period (0 - 365)  # noqa: E501

        :return: The days_paid of this GrossEarningsHistory.  # noqa: E501
        :rtype: int
        """
        return self._days_paid

    @days_paid.setter
    def days_paid(self, days_paid):
        """Sets the days_paid of this GrossEarningsHistory.

        Number of days the employee worked in the pay period (0 - 365)  # noqa: E501

        :param days_paid: The days_paid of this GrossEarningsHistory.  # noqa: E501
        :type: int
        """

        self._days_paid = days_paid

    @property
    def unpaid_weeks(self):
        """Gets the unpaid_weeks of this GrossEarningsHistory.  # noqa: E501

        Number of full weeks the employee didn't work in the pay period (0 - 52)  # noqa: E501

        :return: The unpaid_weeks of this GrossEarningsHistory.  # noqa: E501
        :rtype: int
        """
        return self._unpaid_weeks

    @unpaid_weeks.setter
    def unpaid_weeks(self, unpaid_weeks):
        """Sets the unpaid_weeks of this GrossEarningsHistory.

        Number of full weeks the employee didn't work in the pay period (0 - 52)  # noqa: E501

        :param unpaid_weeks: The unpaid_weeks of this GrossEarningsHistory.  # noqa: E501
        :type: int
        """

        self._unpaid_weeks = unpaid_weeks



# coding: utf-8
"""
    Xero oAuth 2 identity service

    This specifing endpoints related to managing authentication tokens and identity for Xero API  # noqa: E501

    OpenAPI spec version: 2.0.4
    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import datetime
import json
import mimetypes
import os
import re
import tempfile
from decimal import Decimal
from multiprocessing.pool import ThreadPool
from urllib.parse import quote

import xero_python
from xero_python import rest
from xero_python.api_client.configuration import Configuration
from xero_python.api_client.deserializer import deserialize
from xero_python.api_client.serializer import serialize
from xero_python.exceptions import OAuth2TokenGetterError, OAuth2TokenSaverError


class ModelFinder:
    """
    Model finder to find correct model class for given model name
    """

    def __init__(self, models_module):
        self.models = models_module

    def find_model(self, name):
        return getattr(self.models, name)


class ApiClient(object):
    """Generic API client for OpenAPI client library builds.

    OpenAPI generic API client. This client handles the client-
    server communication, and is invariant across implementations. Specifics of
    the methods and models for each application are generated from the OpenAPI
    templates.

    NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech
    Do not edit the class manually.

    :param configuration: .Configuration object for this client
    :param header_name: a header to pass when making calls to the API.
    :param header_value: a header value to pass when making calls to
        the API.
    :param cookie: a cookie to include in the header when making calls
        to the API
    :param pool_threads: The number of threads to use for async requests
        to the API. More threads means more concurrent API requests.
    """

    PRIMITIVE_TYPES = (float, bool, bytes, str, int)
    NATIVE_TYPES_MAPPING = {
        "int": int,
        "long": int,
        "float": float,
        "str": str,
        "bool": bool,
        "date": datetime.date,
        "datetime": datetime.datetime,
        "object": object,
    }
    _pool = None

    def __init__(
        self,
        configuration=None,
        header_name=None,
        header_value=None,
        cookie=None,
        pool_threads=None,
        oauth2_token_saver=None,
        oauth2_token_getter=None,
    ):
        if configuration is None:
            configuration = Configuration()
        self.configuration = configuration
        self.pool_threads = pool_threads

        self.rest_client = rest.RESTClientObject(configuration)
        self.default_headers = {}
        if header_name is not None:
            self.default_headers[header_name] = header_value
        self.cookie = cookie
        # Set default User-Agent.
        self.user_agent = "xero-python ({version})".format(
            version=xero_python.__version__
        )
        self._oauth2_token_saver = oauth2_token_saver
        self._oauth2_token_getter = oauth2_token_getter

    def __del__(self):
        if self._pool:
            self._pool.close()
            self._pool.join()
            self._pool = None

    @property
    def pool(self):
        """Create thread pool on first request
        avoids instantiating unused threadpool for blocking clients.
        """
        if self._pool is None:
            self._pool = ThreadPool(self.pool_threads)
        return self._pool

    @property
    def user_agent(self):
        """User agent for this API client"""
        return self.default_headers["User-Agent"]

    @user_agent.setter
    def user_agent(self, value):
        self.default_headers["User-Agent"] = value

    def set_default_header(self, header_name, header_value):
        self.default_headers[header_name] = header_value

    def __call_api(
        self,
        resource_path,
        method,
        path_params=None,
        query_params=None,
        header_params=None,
        body=None,
        post_params=None,
        files=None,
        response_type=None,
        response_model_finder=None,
        auth_settings=None,
        _return_http_data_only=None,
        collection_formats=None,
        _preload_content=True,
        _request_timeout=None,
    ):

        config = self.configuration

        # header parameters
        header_params = header_params or {}
        header_params.update(self.default_headers)
        if self.cookie:
            header_params["Cookie"] = self.cookie
        if header_params:
            header_params = serialize(header_params)
            header_params = dict(
                self.parameters_to_tuples(header_params, collection_formats)
            )

        # path parameters
        if path_params:
            path_params = serialize(path_params)
            path_params = self.parameters_to_tuples(path_params, collection_formats)
            for k, v in path_params:
                # specified safe chars, encode everything
                resource_path = resource_path.replace(
                    "{%s}" % k, quote(str(v), safe=config.safe_chars_for_path_param)
                )

        # query parameters
        if query_params:
            query_params = serialize(query_params)
            query_params = self.parameters_to_tuples(query_params, collection_formats)

        # post parameters
        if post_params or files:
            post_params = self.prepare_post_parameters(post_params, files)
            post_params = serialize(post_params)
            post_params = self.parameters_to_tuples(post_params, collection_formats)

        # auth setting
        self.update_params_for_auth(header_params, query_params, auth_settings)

        # body
        if body:
            body = serialize(body)

        # request url
        url = resource_path

        # perform request and return response
        response_data = self.request(
            method,
            url,
            query_params=query_params,
            headers=header_params,
            post_params=post_params,
            body=body,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
        )

        self.last_response = response_data

        return_data = response_data
        if _preload_content:
            # deserialize response data
            if response_type:
                return_data = self.deserialize(
                    response_data, response_type, response_model_finder
                )
            else:
                return_data = None

        if _return_http_data_only:
            return return_data
        else:
            return (return_data, response_data.status, response_data.getheaders())

    def deserialize(self, response, response_type, response_model_finder):
        """Deserializes response into an object.

        :param response: RESTResponse object to be deserialized.
        :param response_type: class literal for
            deserialized object, or string of class name.
        :param response_model_finder: ModelFinder instance to find class for
            response_type class literal

        :return: deserialized object.
        """
        # handle file downloading
        # save response body into a tmp file and return the instance
        if response_type == "file":
            return self.__deserialize_file(response)

        # fetch data from response object
        try:
            data = json.loads(response.text, parse_float=Decimal)
        except ValueError:
            data = response.data

        return deserialize(response_type, data, response_model_finder)

    def __deserialize(self, data, klass, model_finder):
        """Deserializes dict, list, str into an object.

        :param data: dict, list or str.
        :param klass: class literal, or string of class name.
        :param model_finder: ModelFinder instance to find class for
            response_type class literal

        :return: object.
        """
        # TODO obsolete, remove
        if data is None:
            return None

        if type(klass) == str:
            if klass.startswith("list["):
                sub_kls = re.match(r"list\[(.*)\]", klass).group(1)
                return [
                    self.__deserialize(sub_data, sub_kls, model_finder)
                    for sub_data in data
                ]

            if klass.startswith("dict("):
                sub_kls = re.match(r"dict\(([^,]*), (.*)\)", klass).group(2)
                return {
                    k: self.__deserialize(v, sub_kls, model_finder)
                    for k, v in data.items()
                }

            # convert str to class
            if klass in self.NATIVE_TYPES_MAPPING:
                klass = self.NATIVE_TYPES_MAPPING[klass]
            else:
                klass = model_finder.find_model(klass)

        if klass in self.PRIMITIVE_TYPES:
            return self.__deserialize_primitive(data, klass, model_finder)
        elif klass == object:
            return self.__deserialize_object(data)
        elif klass == datetime.date:
            return self.__deserialize_date(data)
        elif klass == datetime.datetime:
            return self.__deserialize_datatime(data)
        else:
            return self.__deserialize_model(data, klass, model_finder)

    def call_api(
        self,
        resource_path,
        method,
        path_params=None,
        query_params=None,
        header_params=None,
        body=None,
        post_params=None,
        files=None,
        response_type=None,
        response_model_finder=None,
        auth_settings=None,
        async_req=None,
        _return_http_data_only=None,
        collection_formats=None,
        _preload_content=True,
        _request_timeout=None,
    ):
        """Makes the HTTP request (synchronous) and returns deserialized data.

        To make an async_req request, set the async_req parameter.

        :param resource_path: Path to method endpoint.
        :param method: Method to call.
        :param path_params: Path parameters in the url.
        :param query_params: Query parameters in the url.
        :param header_params: Header parameters to be
            placed in the request header.
        :param body: Request body.
        :param post_params dict: Request post form parameters,
            for `application/x-www-form-urlencoded`, `multipart/form-data`.
        :param auth_settings list: Auth Settings names for the request.
        :param response: Response data type.
        :param files dict: key -> filename, value -> filepath,
            for `multipart/form-data`.
        :param async_req bool: execute request asynchronously
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param collection_formats: dict of collection formats for path, query,
            header, and post parameters.
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return:
            If async_req parameter is True,
            the request will be called asynchronously.
            The method will return the request thread.
            If parameter async_req is False or missing,
            then the method will return the response directly.
        """
        if not async_req:
            return self.__call_api(
                resource_path,
                method,
                path_params,
                query_params,
                header_params,
                body,
                post_params,
                files,
                response_type,
                response_model_finder,
                auth_settings,
                _return_http_data_only,
                collection_formats,
                _preload_content,
                _request_timeout,
            )
        else:
            thread = self.pool.apply_async(
                self.__call_api,
                (
                    resource_path,
                    method,
                    path_params,
                    query_params,
                    header_params,
                    body,
                    post_params,
                    files,
                    response_type,
                    response_model_finder,
                    auth_settings,
                    _return_http_data_only,
                    collection_formats,
                    _preload_content,
                    _request_timeout,
                ),
            )
        return thread

    def request(
        self,
        method,
        url,
        query_params=None,
        headers=None,
        post_params=None,
        body=None,
        _preload_content=True,
        _request_timeout=None,
    ):
        """Makes the HTTP request using RESTClient."""
        if method == "GET":
            return self.rest_client.GET(
                url,
                query_params=query_params,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
                headers=headers,
            )
        elif method == "HEAD":
            return self.rest_client.HEAD(
                url,
                query_params=query_params,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
                headers=headers,
            )
        elif method == "OPTIONS":
            return self.rest_client.OPTIONS(
                url,
                query_params=query_params,
                headers=headers,
                post_params=post_params,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
                body=body,
            )
        elif method == "POST":
            return self.rest_client.POST(
                url,
                query_params=query_params,
                headers=headers,
                post_params=post_params,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
                body=body,
            )
        elif method == "PUT":
            return self.rest_client.PUT(
                url,
                query_params=query_params,
                headers=headers,
                post_params=post_params,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
                body=body,
            )
        elif method == "PATCH":
            return self.rest_client.PATCH(
                url,
                query_params=query_params,
                headers=headers,
                post_params=post_params,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
                body=body,
            )
        elif method == "DELETE":
            return self.rest_client.DELETE(
                url,
                query_params=query_params,
                headers=headers,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
                body=body,
            )
        else:
            raise ValueError(
                "http method must be `GET`, `HEAD`, `OPTIONS`,"
                " `POST`, `PATCH`, `PUT` or `DELETE`."
            )

    def parameters_to_tuples(self, params, collection_formats):
        """Get parameters as list of tuples, formatting collections.

        :param params: Parameters as dict or list of two-tuples
        :param dict collection_formats: Parameter collection formats
        :return: Parameters as list of tuples, collections formatted
        """
        new_params = []
        if collection_formats is None:
            collection_formats = {}
        for k, v in (
            params.items() if isinstance(params, dict) else params
        ):  # noqa: E501
            if k in collection_formats:
                collection_format = collection_formats[k]
                if collection_format == "multi":
                    new_params.extend((k, value) for value in v)
                else:
                    if collection_format == "ssv":
                        delimiter = " "
                    elif collection_format == "tsv":
                        delimiter = "\t"
                    elif collection_format == "pipes":
                        delimiter = "|"
                    else:  # csv is the default
                        delimiter = ","
                    new_params.append((k, delimiter.join(str(value) for value in v)))
            else:
                new_params.append((k, v))
        return new_params

    def prepare_post_parameters(self, post_params=None, files=None):
        """Builds form parameters.

        :param post_params: Normal form parameters.
        :param files: File parameters.
        :return: Form parameters with files.
        """
        params = []

        if post_params:
            params = post_params

        if files:
            for k, v in files.items():
                if not v:
                    continue
                file_names = v if type(v) is list else [v]
                for n in file_names:
                    with open(n, "rb") as f:
                        filename = os.path.basename(f.name)
                        filedata = f.read()
                        mimetype = (
                            mimetypes.guess_type(filename)[0]
                            or "application/octet-stream"
                        )
                        params.append(tuple([k, tuple([filename, filedata, mimetype])]))

        return params

    def select_header_accept(self, accepts):
        """Returns `Accept` based on an array of accepts provided.

        :param accepts: List of headers.
        :return: Accept (e.g. application/json).
        """
        if not accepts:
            return

        accepts = [x.lower() for x in accepts]

        if "application/json" in accepts:
            return "application/json"
        else:
            return ", ".join(accepts)

    def select_header_content_type(self, content_types):
        """Returns `Content-Type` based on an array of content_types provided.

        :param content_types: List of content-types.
        :return: Content-Type (e.g. application/json).
        """
        if not content_types:
            return "application/json"

        content_types = [x.lower() for x in content_types]

        if "application/json" in content_types or "*/*" in content_types:
            return "application/json"
        else:
            return content_types[0]

    def update_params_for_auth(self, headers, querys, auth_settings):
        """Updates header and query params based on authentication setting.

        :param headers: Header parameters dict to be updated.
        :param querys: Query parameters tuple list to be updated.
        :param auth_settings: Authentication setting identifiers list.
        """
        if not auth_settings:
            return

        for auth in auth_settings:
            auth_setting = self.configuration.auth_settings(auth)
            if callable(auth_setting):
                # some auth settings may need http client (eg. auth2 refresh token flow)
                auth_setting = auth_setting(api_client=self)
            if auth_setting:
                if not auth_setting["value"]:
                    continue
                elif auth_setting["in"] == "header":
                    headers[auth_setting["key"]] = auth_setting["value"]
                elif auth_setting["in"] == "query":
                    querys.append((auth_setting["key"], auth_setting["value"]))
                else:
                    raise ValueError(
                        "Authentication token must be in `query` or `header`"
                    )

    def __deserialize_file(self, response):
        """Deserializes body to file

        Saves response body into a file in a temporary folder,
        using the filename from the `Content-Disposition` header if provided.

        :param response:  RESTResponse.
        :return: file path.
        """
        fd, path = tempfile.mkstemp(dir=self.configuration.temp_folder_path)
        os.close(fd)
        os.remove(path)

        content_disposition = response.getheader("Content-Disposition")
        if content_disposition:
            filename = re.search(
                r'filename=[\'"]?([^\'"\s]+)[\'"]?', content_disposition
            ).group(1)
            path = os.path.join(os.path.dirname(path), filename)

        with open(path, "wb") as f:
            f.write(response.data)

        return path

    def __deserialize_primitive(self, data, klass, model_finder):
        """Deserializes string to primitive type.

        :param data: str.
        :param klass: class literal.
        :param model_finder: ModelFinder instance to find class for
            response_type class literal

        :return: int, long, float, str, bool.
        """
        # TODO obsolete, remove
        try:
            return klass(data)
        except UnicodeEncodeError:
            return str(data)
        except TypeError:
            return data

    def __deserialize_object(self, value):
        """Return an original value.

        :return: object.
        """
        # TODO obsolete, remove
        return value

    def __deserialize_date(self, string):
        """Deserializes string to date.

        :param string: str.
        :return: date.
        """
        # TODO obsolete, remove
        try:
            from dateutil.parser import parse

            return parse(string).date()
        except ImportError:
            return string
        except ValueError:
            raise rest.ApiException(
                status=0, reason="Failed to parse `{0}` as date object".format(string)
            )

    def __deserialize_datatime(self, string):
        """Deserializes string to datetime.

        The string should be in iso8601 datetime format.

        :param string: str.
        :return: datetime.
        """
        # TODO obsolete, remove
        try:
            from dateutil.parser import parse

            return parse(string)
        except ImportError:
            return string
        except ValueError:
            raise rest.ApiException(
                status=0,
                reason=("Failed to parse `{0}` as datetime object".format(string)),
            )

    def __deserialize_model(self, data, klass, model_finder):
        """Deserializes list or dict to model.

        :param data: dict, list.
        :param klass: class literal.
        :param model_finder: ModelFinder instance to find class for
            response_type class literal

        :return: model object.
        """
        # TODO obsolete, remove
        if not klass.openapi_types and not hasattr(klass, "get_real_child_model"):
            return data

        kwargs = {}
        if klass.openapi_types is not None:
            for attr, attr_type in klass.openapi_types.items():
                if (
                    data is not None
                    and klass.attribute_map[attr] in data
                    and isinstance(data, (list, dict))
                ):
                    value = data[klass.attribute_map[attr]]
                    kwargs[attr] = self.__deserialize(value, attr_type, model_finder)

        instance = klass(**kwargs)

        if hasattr(instance, "get_real_child_model"):
            klass_name = instance.get_real_child_model(data)
            if klass_name:
                instance = self.__deserialize(data, klass_name, model_finder)
        return instance

    def get_oauth2_token(self):
        """
        Get oauth2 token dictionary
        :return: dict
        """
        if not self._oauth2_token_getter:
            raise OAuth2TokenGetterError(
                "Invalid oauth2_token_getter={!r} function".format(
                    self._oauth2_token_getter
                )
            )
        return self._oauth2_token_getter()

    def set_oauth2_token(self, token):
        """
        Set oauth2 token
        :param dict token: standard token dictionary
        """
        if not self._oauth2_token_saver:
            raise OAuth2TokenSaverError(
                "Invalid oauth2_token_saver={!r} function".format(
                    self._oauth2_token_saver
                )
            )
        self._oauth2_token_saver(token)

    def refresh_oauth2_token(self):
        """
        Force refresh oauth2 token
        :return: new oauth2 token
        """
        oauth2_token = self.configuration.oauth2_token
        oauth2_token.update_token(**self.get_oauth2_token())
        if oauth2_token.refresh_access_token(self):
            return self.get_oauth2_token()

    def oauth2_token_getter(self, token_getter):
        """
        A decorator to register a callback function for getting oauth2 token

        It is necessary for token synchronisation across the application code.

        :param token_getter: the callback function returning oauth2 token dictionary.
        :return: token_getter to allow this method be used as decorator
        """
        self._oauth2_token_getter = token_getter
        return token_getter

    def oauth2_token_saver(self, token_saver):
        """
        A decorator to register a callback function for saving refreshed
        token while the old token has expired

        It is necessary for using the automatic refresh mechanism.

        :param token_saver: the callback function accepting `token` argument.
        :return: token_saver to allow this method be used as decorator
        """
        self._oauth2_token_saver = token_saver
        return token_saver



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class Employees(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"employees": "list[Employee]"}

    attribute_map = {"employees": "Employees"}

    def __init__(self, employees=None):  # noqa: E501
        """Employees - a model defined in OpenAPI"""  # noqa: E501

        self._employees = None
        self.discriminator = None

        if employees is not None:
            self.employees = employees

    @property
    def employees(self):
        """Gets the employees of this Employees.  # noqa: E501


        :return: The employees of this Employees.  # noqa: E501
        :rtype: list[Employee]
        """
        return self._employees

    @employees.setter
    def employees(self, employees):
        """Sets the employees of this Employees.


        :param employees: The employees of this Employees.  # noqa: E501
        :type: list[Employee]
        """

        self._employees = employees



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class LineItem(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "line_item_id": "str",
        "description": "str",
        "quantity": "float",
        "unit_amount": "float",
        "item_code": "str",
        "account_code": "str",
        "tax_type": "str",
        "tax_amount": "float",
        "line_amount": "float",
        "tracking": "list[LineItemTracking]",
        "discount_rate": "float",
        "discount_amount": "float",
        "repeating_invoice_id": "str",
    }

    attribute_map = {
        "line_item_id": "LineItemID",
        "description": "Description",
        "quantity": "Quantity",
        "unit_amount": "UnitAmount",
        "item_code": "ItemCode",
        "account_code": "AccountCode",
        "tax_type": "TaxType",
        "tax_amount": "TaxAmount",
        "line_amount": "LineAmount",
        "tracking": "Tracking",
        "discount_rate": "DiscountRate",
        "discount_amount": "DiscountAmount",
        "repeating_invoice_id": "RepeatingInvoiceID",
    }

    def __init__(
        self,
        line_item_id=None,
        description=None,
        quantity=None,
        unit_amount=None,
        item_code=None,
        account_code=None,
        tax_type=None,
        tax_amount=None,
        line_amount=None,
        tracking=None,
        discount_rate=None,
        discount_amount=None,
        repeating_invoice_id=None,
    ):  # noqa: E501
        """LineItem - a model defined in OpenAPI"""  # noqa: E501

        self._line_item_id = None
        self._description = None
        self._quantity = None
        self._unit_amount = None
        self._item_code = None
        self._account_code = None
        self._tax_type = None
        self._tax_amount = None
        self._line_amount = None
        self._tracking = None
        self._discount_rate = None
        self._discount_amount = None
        self._repeating_invoice_id = None
        self.discriminator = None

        if line_item_id is not None:
            self.line_item_id = line_item_id
        if description is not None:
            self.description = description
        if quantity is not None:
            self.quantity = quantity
        if unit_amount is not None:
            self.unit_amount = unit_amount
        if item_code is not None:
            self.item_code = item_code
        if account_code is not None:
            self.account_code = account_code
        if tax_type is not None:
            self.tax_type = tax_type
        if tax_amount is not None:
            self.tax_amount = tax_amount
        if line_amount is not None:
            self.line_amount = line_amount
        if tracking is not None:
            self.tracking = tracking
        if discount_rate is not None:
            self.discount_rate = discount_rate
        if discount_amount is not None:
            self.discount_amount = discount_amount
        if repeating_invoice_id is not None:
            self.repeating_invoice_id = repeating_invoice_id

    @property
    def line_item_id(self):
        """Gets the line_item_id of this LineItem.  # noqa: E501

        LineItem unique ID  # noqa: E501

        :return: The line_item_id of this LineItem.  # noqa: E501
        :rtype: str
        """
        return self._line_item_id

    @line_item_id.setter
    def line_item_id(self, line_item_id):
        """Sets the line_item_id of this LineItem.

        LineItem unique ID  # noqa: E501

        :param line_item_id: The line_item_id of this LineItem.  # noqa: E501
        :type: str
        """

        self._line_item_id = line_item_id

    @property
    def description(self):
        """Gets the description of this LineItem.  # noqa: E501

        Description needs to be at least 1 char long. A line item with just a description (i.e no unit amount or quantity) can be created by specifying just a <Description> element that contains at least 1 character  # noqa: E501

        :return: The description of this LineItem.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this LineItem.

        Description needs to be at least 1 char long. A line item with just a description (i.e no unit amount or quantity) can be created by specifying just a <Description> element that contains at least 1 character  # noqa: E501

        :param description: The description of this LineItem.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def quantity(self):
        """Gets the quantity of this LineItem.  # noqa: E501

        LineItem Quantity  # noqa: E501

        :return: The quantity of this LineItem.  # noqa: E501
        :rtype: float
        """
        return self._quantity

    @quantity.setter
    def quantity(self, quantity):
        """Sets the quantity of this LineItem.

        LineItem Quantity  # noqa: E501

        :param quantity: The quantity of this LineItem.  # noqa: E501
        :type: float
        """

        self._quantity = quantity

    @property
    def unit_amount(self):
        """Gets the unit_amount of this LineItem.  # noqa: E501

        LineItem Unit Amount  # noqa: E501

        :return: The unit_amount of this LineItem.  # noqa: E501
        :rtype: float
        """
        return self._unit_amount

    @unit_amount.setter
    def unit_amount(self, unit_amount):
        """Sets the unit_amount of this LineItem.

        LineItem Unit Amount  # noqa: E501

        :param unit_amount: The unit_amount of this LineItem.  # noqa: E501
        :type: float
        """

        self._unit_amount = unit_amount

    @property
    def item_code(self):
        """Gets the item_code of this LineItem.  # noqa: E501

        See Items  # noqa: E501

        :return: The item_code of this LineItem.  # noqa: E501
        :rtype: str
        """
        return self._item_code

    @item_code.setter
    def item_code(self, item_code):
        """Sets the item_code of this LineItem.

        See Items  # noqa: E501

        :param item_code: The item_code of this LineItem.  # noqa: E501
        :type: str
        """

        self._item_code = item_code

    @property
    def account_code(self):
        """Gets the account_code of this LineItem.  # noqa: E501

        See Accounts  # noqa: E501

        :return: The account_code of this LineItem.  # noqa: E501
        :rtype: str
        """
        return self._account_code

    @account_code.setter
    def account_code(self, account_code):
        """Sets the account_code of this LineItem.

        See Accounts  # noqa: E501

        :param account_code: The account_code of this LineItem.  # noqa: E501
        :type: str
        """

        self._account_code = account_code

    @property
    def tax_type(self):
        """Gets the tax_type of this LineItem.  # noqa: E501

        The tax type from TaxRates  # noqa: E501

        :return: The tax_type of this LineItem.  # noqa: E501
        :rtype: str
        """
        return self._tax_type

    @tax_type.setter
    def tax_type(self, tax_type):
        """Sets the tax_type of this LineItem.

        The tax type from TaxRates  # noqa: E501

        :param tax_type: The tax_type of this LineItem.  # noqa: E501
        :type: str
        """

        self._tax_type = tax_type

    @property
    def tax_amount(self):
        """Gets the tax_amount of this LineItem.  # noqa: E501

        The tax amount is auto calculated as a percentage of the line amount (see below) based on the tax rate. This value can be overriden if the calculated <TaxAmount> is not correct.  # noqa: E501

        :return: The tax_amount of this LineItem.  # noqa: E501
        :rtype: float
        """
        return self._tax_amount

    @tax_amount.setter
    def tax_amount(self, tax_amount):
        """Sets the tax_amount of this LineItem.

        The tax amount is auto calculated as a percentage of the line amount (see below) based on the tax rate. This value can be overriden if the calculated <TaxAmount> is not correct.  # noqa: E501

        :param tax_amount: The tax_amount of this LineItem.  # noqa: E501
        :type: float
        """

        self._tax_amount = tax_amount

    @property
    def line_amount(self):
        """Gets the line_amount of this LineItem.  # noqa: E501

        If you wish to omit either of the <Quantity> or <UnitAmount> you can provide a LineAmount and Xero will calculate the missing amount for you. The line amount reflects the discounted price if a DiscountRate has been used . i.e LineAmount = Quantity * Unit Amount * ((100 – DiscountRate)/100)  # noqa: E501

        :return: The line_amount of this LineItem.  # noqa: E501
        :rtype: float
        """
        return self._line_amount

    @line_amount.setter
    def line_amount(self, line_amount):
        """Sets the line_amount of this LineItem.

        If you wish to omit either of the <Quantity> or <UnitAmount> you can provide a LineAmount and Xero will calculate the missing amount for you. The line amount reflects the discounted price if a DiscountRate has been used . i.e LineAmount = Quantity * Unit Amount * ((100 – DiscountRate)/100)  # noqa: E501

        :param line_amount: The line_amount of this LineItem.  # noqa: E501
        :type: float
        """

        self._line_amount = line_amount

    @property
    def tracking(self):
        """Gets the tracking of this LineItem.  # noqa: E501

        Optional Tracking Category – see Tracking.  Any LineItem can have a  maximum of 2 <TrackingCategory> elements.  # noqa: E501

        :return: The tracking of this LineItem.  # noqa: E501
        :rtype: list[LineItemTracking]
        """
        return self._tracking

    @tracking.setter
    def tracking(self, tracking):
        """Sets the tracking of this LineItem.

        Optional Tracking Category – see Tracking.  Any LineItem can have a  maximum of 2 <TrackingCategory> elements.  # noqa: E501

        :param tracking: The tracking of this LineItem.  # noqa: E501
        :type: list[LineItemTracking]
        """

        self._tracking = tracking

    @property
    def discount_rate(self):
        """Gets the discount_rate of this LineItem.  # noqa: E501

        Percentage discount being applied to a line item (only supported on  ACCREC invoices – ACC PAY invoices and credit notes in Xero do not support discounts  # noqa: E501

        :return: The discount_rate of this LineItem.  # noqa: E501
        :rtype: float
        """
        return self._discount_rate

    @discount_rate.setter
    def discount_rate(self, discount_rate):
        """Sets the discount_rate of this LineItem.

        Percentage discount being applied to a line item (only supported on  ACCREC invoices – ACC PAY invoices and credit notes in Xero do not support discounts  # noqa: E501

        :param discount_rate: The discount_rate of this LineItem.  # noqa: E501
        :type: float
        """

        self._discount_rate = discount_rate

    @property
    def discount_amount(self):
        """Gets the discount_amount of this LineItem.  # noqa: E501

        Discount amount being applied to a line item. Only supported on ACCREC invoices - ACCPAY invoices and credit notes in Xero do not support discounts.  # noqa: E501

        :return: The discount_amount of this LineItem.  # noqa: E501
        :rtype: float
        """
        return self._discount_amount

    @discount_amount.setter
    def discount_amount(self, discount_amount):
        """Sets the discount_amount of this LineItem.

        Discount amount being applied to a line item. Only supported on ACCREC invoices - ACCPAY invoices and credit notes in Xero do not support discounts.  # noqa: E501

        :param discount_amount: The discount_amount of this LineItem.  # noqa: E501
        :type: float
        """

        self._discount_amount = discount_amount

    @property
    def repeating_invoice_id(self):
        """Gets the repeating_invoice_id of this LineItem.  # noqa: E501

        The Xero identifier for a Repeating Invoice  # noqa: E501

        :return: The repeating_invoice_id of this LineItem.  # noqa: E501
        :rtype: str
        """
        return self._repeating_invoice_id

    @repeating_invoice_id.setter
    def repeating_invoice_id(self, repeating_invoice_id):
        """Sets the repeating_invoice_id of this LineItem.

        The Xero identifier for a Repeating Invoice  # noqa: E501

        :param repeating_invoice_id: The repeating_invoice_id of this LineItem.  # noqa: E501
        :type: str
        """

        self._repeating_invoice_id = repeating_invoice_id



# -*- coding: utf-8 -*-
import pytest
import urllib3


def test_the_tests():
    """
    to test the tests configuration
    """

    assert True is True


@pytest.mark.vcr()
def test_vcr():
    """
    Test VCR.py records and plays http requests properly
    """
    http_manager = urllib3.PoolManager()
    response = http_manager.request(
        "GET", "https://developer.xero.com/documentation/oauth2/auth-flow"
    )
    assert response.status == 200
    assert "Xero is a multi-tenanted platform." in response.data.decode("utf-8")



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class ManualJournalLine(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "line_amount": "float",
        "account_code": "str",
        "account_id": "str",
        "description": "str",
        "tax_type": "str",
        "tracking": "list[TrackingCategory]",
        "tax_amount": "float",
        "is_blank": "bool",
    }

    attribute_map = {
        "line_amount": "LineAmount",
        "account_code": "AccountCode",
        "account_id": "AccountID",
        "description": "Description",
        "tax_type": "TaxType",
        "tracking": "Tracking",
        "tax_amount": "TaxAmount",
        "is_blank": "IsBlank",
    }

    def __init__(
        self,
        line_amount=None,
        account_code=None,
        account_id=None,
        description=None,
        tax_type=None,
        tracking=None,
        tax_amount=None,
        is_blank=None,
    ):  # noqa: E501
        """ManualJournalLine - a model defined in OpenAPI"""  # noqa: E501

        self._line_amount = None
        self._account_code = None
        self._account_id = None
        self._description = None
        self._tax_type = None
        self._tracking = None
        self._tax_amount = None
        self._is_blank = None
        self.discriminator = None

        if line_amount is not None:
            self.line_amount = line_amount
        if account_code is not None:
            self.account_code = account_code
        if account_id is not None:
            self.account_id = account_id
        if description is not None:
            self.description = description
        if tax_type is not None:
            self.tax_type = tax_type
        if tracking is not None:
            self.tracking = tracking
        if tax_amount is not None:
            self.tax_amount = tax_amount
        if is_blank is not None:
            self.is_blank = is_blank

    @property
    def line_amount(self):
        """Gets the line_amount of this ManualJournalLine.  # noqa: E501

        total for line. Debits are positive, credits are negative value  # noqa: E501

        :return: The line_amount of this ManualJournalLine.  # noqa: E501
        :rtype: float
        """
        return self._line_amount

    @line_amount.setter
    def line_amount(self, line_amount):
        """Sets the line_amount of this ManualJournalLine.

        total for line. Debits are positive, credits are negative value  # noqa: E501

        :param line_amount: The line_amount of this ManualJournalLine.  # noqa: E501
        :type: float
        """

        self._line_amount = line_amount

    @property
    def account_code(self):
        """Gets the account_code of this ManualJournalLine.  # noqa: E501

        See Accounts  # noqa: E501

        :return: The account_code of this ManualJournalLine.  # noqa: E501
        :rtype: str
        """
        return self._account_code

    @account_code.setter
    def account_code(self, account_code):
        """Sets the account_code of this ManualJournalLine.

        See Accounts  # noqa: E501

        :param account_code: The account_code of this ManualJournalLine.  # noqa: E501
        :type: str
        """

        self._account_code = account_code

    @property
    def account_id(self):
        """Gets the account_id of this ManualJournalLine.  # noqa: E501

        See Accounts  # noqa: E501

        :return: The account_id of this ManualJournalLine.  # noqa: E501
        :rtype: str
        """
        return self._account_id

    @account_id.setter
    def account_id(self, account_id):
        """Sets the account_id of this ManualJournalLine.

        See Accounts  # noqa: E501

        :param account_id: The account_id of this ManualJournalLine.  # noqa: E501
        :type: str
        """

        self._account_id = account_id

    @property
    def description(self):
        """Gets the description of this ManualJournalLine.  # noqa: E501

        Description for journal line  # noqa: E501

        :return: The description of this ManualJournalLine.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this ManualJournalLine.

        Description for journal line  # noqa: E501

        :param description: The description of this ManualJournalLine.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def tax_type(self):
        """Gets the tax_type of this ManualJournalLine.  # noqa: E501

        The tax type from TaxRates  # noqa: E501

        :return: The tax_type of this ManualJournalLine.  # noqa: E501
        :rtype: str
        """
        return self._tax_type

    @tax_type.setter
    def tax_type(self, tax_type):
        """Sets the tax_type of this ManualJournalLine.

        The tax type from TaxRates  # noqa: E501

        :param tax_type: The tax_type of this ManualJournalLine.  # noqa: E501
        :type: str
        """

        self._tax_type = tax_type

    @property
    def tracking(self):
        """Gets the tracking of this ManualJournalLine.  # noqa: E501

        Optional Tracking Category – see Tracking. Any JournalLine can have a maximum of 2 <TrackingCategory> elements.  # noqa: E501

        :return: The tracking of this ManualJournalLine.  # noqa: E501
        :rtype: list[TrackingCategory]
        """
        return self._tracking

    @tracking.setter
    def tracking(self, tracking):
        """Sets the tracking of this ManualJournalLine.

        Optional Tracking Category – see Tracking. Any JournalLine can have a maximum of 2 <TrackingCategory> elements.  # noqa: E501

        :param tracking: The tracking of this ManualJournalLine.  # noqa: E501
        :type: list[TrackingCategory]
        """

        self._tracking = tracking

    @property
    def tax_amount(self):
        """Gets the tax_amount of this ManualJournalLine.  # noqa: E501

        The calculated tax amount based on the TaxType and LineAmount  # noqa: E501

        :return: The tax_amount of this ManualJournalLine.  # noqa: E501
        :rtype: float
        """
        return self._tax_amount

    @tax_amount.setter
    def tax_amount(self, tax_amount):
        """Sets the tax_amount of this ManualJournalLine.

        The calculated tax amount based on the TaxType and LineAmount  # noqa: E501

        :param tax_amount: The tax_amount of this ManualJournalLine.  # noqa: E501
        :type: float
        """

        self._tax_amount = tax_amount

    @property
    def is_blank(self):
        """Gets the is_blank of this ManualJournalLine.  # noqa: E501

        is the line blank  # noqa: E501

        :return: The is_blank of this ManualJournalLine.  # noqa: E501
        :rtype: bool
        """
        return self._is_blank

    @is_blank.setter
    def is_blank(self, is_blank):
        """Sets the is_blank of this ManualJournalLine.

        is the line blank  # noqa: E501

        :param is_blank: The is_blank of this ManualJournalLine.  # noqa: E501
        :type: bool
        """

        self._is_blank = is_blank



# -*- coding: utf-8 -*-
from contextlib import contextmanager
from datetime import date, datetime, timedelta
from decimal import Decimal
from unittest import mock

import pytest
from dateutil import tz

from xero_python.api_client.serializer import (
    data_type,
    serialize,
    serialize_routing,
    serialize_dict,
    serialize_pass_through,
    serialize_list,
    serialize_tuple,
    serialize_float,
    serialize_datetime_ms,
    serialize_datetime,
    serialize_date_ms,
    serialize_base_model,
    serialize_enum_model,
)
from xero_python.models import BaseModel
from .test_deserializer import Shape


@contextmanager
def mock_serialize(name="serialize", side_effect=None):
    def sub_response(value, explicit_type=None):
        return value

    with mock.patch(
        "xero_python.api_client.serializer.{}".format(name),
        side_effect=side_effect or sub_response,
    ) as serialize:
        yield serialize


# data_type tests
@pytest.mark.parametrize(
    "value,explicit_type,expected",
    [
        ("value", None, "str"),
        (1, None, "int"),
        (True, None, "bool"),
        (1.0, None, "float"),
        (date.today(), None, "date"),
        (date.today(), "date[ms-format]", "date[ms-format]"),
        (datetime.today(), None, "datetime"),
        (datetime.today(), "datetime[ms-format]", "datetime[ms-format]"),
        (BaseModel(), None, "BaseModel"),
    ],
)
def test_data_type(value, explicit_type, expected):
    # given value and optional explicit data type
    # when finding serialize data type
    value_type = data_type(value, explicit_type)
    # then value_type to be as expected
    assert value_type == expected


# serialize_routing tests
@pytest.mark.parametrize(
    "value,explicit_type",
    [
        ("value", None),
        (1, None),
        (True, None),
        (1.0, None),
        (date.today(), None),
        (date.today(), "date[ms-format]"),
        (datetime.today(), None),
        (datetime.today(), "datetime[ms-format]"),
        (BaseModel(), None),
    ],
)
def test_serialize_routing(value, explicit_type):
    # when serialize test user
    def sub_response(value, explicit_type):
        return str(value)

    with mock_serialize("data_type", side_effect=sub_response) as data_type:
        key = serialize_routing(value, explicit_type)

    # then expect data_type called
    data_type.assert_called_once_with(value, explicit_type)
    # then expected result to returned from data_type()
    assert key == str(value)


@pytest.mark.parametrize(
    "value,explicit_type",
    [(dict(), None), (None, "dict"), (None, "dict[str]"), (None, "dict[User]")],
)
def test_serialize_routing_dict(value, explicit_type):
    # given dict value or explicit type
    # when finding serialize routing key
    key = serialize_routing(value, explicit_type)
    # then expected key to be 'dict'
    assert key == "dict"


@pytest.mark.parametrize(
    "value,explicit_type",
    [([], None), (None, "list"), (None, "list[str]"), (None, "list[User]")],
)
def test_serialize_routing_list(value, explicit_type):
    # given list value or explicit type
    # when finding serialize routing key
    key = serialize_routing(value, explicit_type)
    # then expected key to be 'list'
    assert key == "list"


@pytest.mark.parametrize(
    "value,explicit_type",
    [(tuple(), None), (None, "tuple"), (None, "tuple[str]"), (None, "tuple[User]")],
)
def test_serialize_routing_tuple(value, explicit_type):
    # given tuple value or explicit type
    # when finding serialize routing key
    key = serialize_routing(value, explicit_type)
    # then expected key to be 'tuple'
    assert key == "tuple"


# serialize tests
def test_serialize():
    # given default serialize function
    default_serialize = serialize.registry[""]  # get default implementation

    class User(BaseModel):
        """
        test api model
        """

    value = User()
    # when serialize test user
    with mock_serialize("serialize_model") as serialize_model:
        result = default_serialize(value)

    # then expect serialize_model called
    serialize_model.assert_called_once_with(value)
    # then expected result to returned from serialize_model()
    assert result is value


@pytest.mark.parametrize(
    "value,explicit_type", [(None, "User"), (BaseModel(), "UserModel")]
)
def test_serialize_non_model(value, explicit_type):
    # given default serialize function
    default_serialize = serialize.registry[""]  # get default implementation
    # when serialize non model value, ValueError expected
    with pytest.raises(ValueError):
        default_serialize(value, explicit_type)


# serialize_dict tests
@pytest.mark.parametrize(
    "value,explicit_type,sub_type",
    [
        ({}, None, None),
        ({"key": "value", "key2": ""}, "dict", None),
        ({"key": "1"}, "dict[int]", "int"),
    ],
)
def test_serialize_dict(value, explicit_type, sub_type):
    # given dict value
    # when serialize dict of items
    with mock_serialize() as serialize:
        result = serialize_dict(value, explicit_type)
    # then expected json serializable dict result
    assert isinstance(result, dict)
    assert len(result) == len(value)
    assert result == value
    for sub_value in value.values():
        serialize.assert_any_call(sub_value, sub_type)


def test_serialize_dict_sub_type_error():
    # given invalid sub type key
    explicit_type = "dict[number"
    value = {"key": "1"}
    # when deserialize dict of items
    with pytest.raises(ValueError):
        serialize_dict(value, explicit_type)


# serialize_list tests
@pytest.mark.parametrize(
    "value,explicit_type,sub_type",
    [
        ([], None, None),
        (["value", "val2", ""], "list", None),
        (["0", "1"], "list[int]", "int"),
    ],
)
def test_serialize_list(value, explicit_type, sub_type):
    # given list value
    # when serialize list of items
    with mock_serialize() as serialize:
        result = serialize_list(value, explicit_type)
    # then expected json serializable list result
    assert isinstance(result, list)
    assert len(result) == len(value)
    assert result == value
    for sub_value in value:
        serialize.assert_any_call(sub_value, sub_type)


def test_serialize_list_sub_type_error():
    # given invalid sub type key
    explicit_type = "list[number"
    value = ["key", "1"]
    # when deserialize list of items
    with pytest.raises(ValueError):
        serialize_list(value, explicit_type)


# serialize_tuple tests
@pytest.mark.parametrize(
    "value,explicit_type,sub_type",
    [
        (tuple(), None, None),
        (("value", "val2", ""), "tuple", None),
        (("0", "1"), "tuple[int]", "int"),
    ],
)
def test_serialize_tuple(value, explicit_type, sub_type):
    # given tuple value
    # when serialize tuple of items
    with mock_serialize() as serialize:
        result = serialize_tuple(value, explicit_type)
    # then expected json serializable tuple result
    assert isinstance(result, tuple)
    assert len(result) == len(value)
    assert result == value
    for sub_value in value:
        serialize.assert_any_call(sub_value, sub_type)


def test_serialize_tuple_sub_type_error():
    # given invalid sub type key
    explicit_type = "tuple[number"
    value = ("key", "1")
    # when deserialize tuple of items
    with pytest.raises(ValueError):
        serialize_tuple(value, explicit_type)


# serialize_pass_through tests
@pytest.mark.parametrize("value", ["string", True, -12])
def test_serialize_pass_through(value):
    # given json serializable value
    # when serialize_pass_through
    result = serialize_pass_through(value)
    # then expected result to be original value
    assert result is value


# serialize_float tests
@pytest.mark.parametrize("value", [0.0001, Decimal("0.1")])
def test_serialize_float(value):
    # given float or Decimal value
    # when serialize_float
    result = serialize_float(value)
    # then expected result to be 'float' value
    assert isinstance(result, float)
    assert result == float(value)


# serialize_datetime tests
@pytest.mark.parametrize(
    "value,expected",
    [
        (date(2020, 4, 3), "2020-04-03"),
        (date(2020, 2, 28), "2020-02-28"),
        (date(2019, 8, 1), "2019-08-01"),
        (datetime(2020, 4, 3, 5, 35, 6), "2020-04-03T05:35:06"),
        (datetime(2020, 2, 28, 11, 2, 54), "2020-02-28T11:02:54"),
        (datetime(2019, 12, 7, 18, 46, 19, 518784), "2019-12-07T18:46:19.518784"),
        (
            datetime(2019, 12, 10, 12, 59, 59, tzinfo=tz.UTC),
            "2019-12-10T12:59:59+00:00",
        ),
        (
            datetime(2019, 12, 7, 18, 46, 19, tzinfo=tz.gettz("Pacific/Auckland")),
            "2019-12-07T18:46:19+13:00",
        ),
    ],
)
def test_serialize_datetime(value, expected):
    # given date value
    # when serialize date or datetime
    result = serialize_datetime(value)
    # then expect result to ISO-8601 string
    assert isinstance(result, str)
    assert result == expected


# serialize_datetime_ms tests
@pytest.mark.parametrize(
    "value,expected",
    [
        (datetime.fromtimestamp(0.0), "/Date(0)/"),
        (datetime.fromtimestamp(1439424000.0), "/Date(1439424000000)/"),
        (datetime.fromtimestamp(1439434356.790), "/Date(1439434356790)/"),
        (datetime(2015, 8, 13, tzinfo=tz.UTC), "/Date(1439424000000+0000)/"),
        (
            datetime(2016, 10, 13, 20, 13, 36, 437000, tzinfo=tz.UTC),
            "/Date(1476389616437+0000)/",
        ),
        (
            datetime(
                2016,
                10,
                14,
                9,
                13,
                36,
                437000,
                tzinfo=tz.tzoffset(None, timedelta(hours=13)),
            ),
            "/Date(1476389616437+1300)/",
        ),
    ],
)
def test_serialize_datetime_ms(value, expected):
    # given datetime value
    # when serialize datetime
    result = serialize_datetime_ms(value)
    # then expect result to be date string in MS json format
    assert isinstance(result, str)
    assert result == expected


# serialize_date_ms tests
@pytest.mark.parametrize(
    "value,expected",
    [
        (date(1970, 1, 1), "/Date(0)/"),
        (date(1980, 1, 2), "/Date(315619200000)/"),
        (date(2017, 11, 28), "/Date(1511827200000)/"),
        (date(2019, 2, 23), "/Date(1550880000000)/"),
    ],
)
def test_serialize_date_ms(value, expected):
    # given datetime value
    # when serialize date
    result = serialize_date_ms(value)
    # then expect result to be date string in MS json format
    assert isinstance(result, str)
    assert result == expected


# serialize_base_model tests
def test_serialize_base_model():
    # given test model
    class Model(BaseModel):
        openapi_types = {"value": "int", "description": "str"}
        attribute_map = {"value": "Value", "description": "Short"}

        def __init__(self, **kwargs):
            for name, value in kwargs.items():
                setattr(self, name, value)

    value = Model(value=132, description="short description")
    # when serialize model
    with mock_serialize() as serialize:
        result = serialize_base_model(value)
    # then expect result to dictionary
    assert isinstance(result, dict)
    assert result == {"Value": 132, "Short": "short description"}
    serialize.assert_any_call(132, "int")
    serialize.assert_any_call("short description", "str")


@pytest.mark.parametrize(
    "value,expected",
    [(Shape.SQUARE, "square"), (Shape.DIAMOND, "diamond"), (Shape.CIRCLE, "circle")],
)
def test_serialize_enum_model(value, expected):
    # given test enum model instance
    # when serialize model
    result = serialize_enum_model(value)
    # then correct enum value expected
    assert isinstance(result, str)
    assert result == expected



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class BatchPaymentDetails(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "bank_account_number": "str",
        "bank_account_name": "str",
        "details": "str",
        "code": "str",
        "reference": "str",
    }

    attribute_map = {
        "bank_account_number": "BankAccountNumber",
        "bank_account_name": "BankAccountName",
        "details": "Details",
        "code": "Code",
        "reference": "Reference",
    }

    def __init__(
        self,
        bank_account_number=None,
        bank_account_name=None,
        details=None,
        code=None,
        reference=None,
    ):  # noqa: E501
        """BatchPaymentDetails - a model defined in OpenAPI"""  # noqa: E501

        self._bank_account_number = None
        self._bank_account_name = None
        self._details = None
        self._code = None
        self._reference = None
        self.discriminator = None

        if bank_account_number is not None:
            self.bank_account_number = bank_account_number
        if bank_account_name is not None:
            self.bank_account_name = bank_account_name
        if details is not None:
            self.details = details
        if code is not None:
            self.code = code
        if reference is not None:
            self.reference = reference

    @property
    def bank_account_number(self):
        """Gets the bank_account_number of this BatchPaymentDetails.  # noqa: E501

        Bank account number for use with Batch Payments  # noqa: E501

        :return: The bank_account_number of this BatchPaymentDetails.  # noqa: E501
        :rtype: str
        """
        return self._bank_account_number

    @bank_account_number.setter
    def bank_account_number(self, bank_account_number):
        """Sets the bank_account_number of this BatchPaymentDetails.

        Bank account number for use with Batch Payments  # noqa: E501

        :param bank_account_number: The bank_account_number of this BatchPaymentDetails.  # noqa: E501
        :type: str
        """

        self._bank_account_number = bank_account_number

    @property
    def bank_account_name(self):
        """Gets the bank_account_name of this BatchPaymentDetails.  # noqa: E501

        Name of bank for use with Batch Payments  # noqa: E501

        :return: The bank_account_name of this BatchPaymentDetails.  # noqa: E501
        :rtype: str
        """
        return self._bank_account_name

    @bank_account_name.setter
    def bank_account_name(self, bank_account_name):
        """Sets the bank_account_name of this BatchPaymentDetails.

        Name of bank for use with Batch Payments  # noqa: E501

        :param bank_account_name: The bank_account_name of this BatchPaymentDetails.  # noqa: E501
        :type: str
        """

        self._bank_account_name = bank_account_name

    @property
    def details(self):
        """Gets the details of this BatchPaymentDetails.  # noqa: E501

        (Non-NZ Only) These details are sent to the org’s bank as a reference for the batch payment transaction. They will also show with the batch payment transaction in the bank reconciliation Find & Match screen. Depending on your individual bank, the detail may also show on the bank statement imported into Xero. Maximum field length = 18  # noqa: E501

        :return: The details of this BatchPaymentDetails.  # noqa: E501
        :rtype: str
        """
        return self._details

    @details.setter
    def details(self, details):
        """Sets the details of this BatchPaymentDetails.

        (Non-NZ Only) These details are sent to the org’s bank as a reference for the batch payment transaction. They will also show with the batch payment transaction in the bank reconciliation Find & Match screen. Depending on your individual bank, the detail may also show on the bank statement imported into Xero. Maximum field length = 18  # noqa: E501

        :param details: The details of this BatchPaymentDetails.  # noqa: E501
        :type: str
        """

        self._details = details

    @property
    def code(self):
        """Gets the code of this BatchPaymentDetails.  # noqa: E501

        (NZ Only) Optional references for the batch payment transaction. It will also show with the batch payment transaction in the bank reconciliation Find & Match screen. Depending on your individual bank, the detail may also show on the bank statement you import into Xero.  # noqa: E501

        :return: The code of this BatchPaymentDetails.  # noqa: E501
        :rtype: str
        """
        return self._code

    @code.setter
    def code(self, code):
        """Sets the code of this BatchPaymentDetails.

        (NZ Only) Optional references for the batch payment transaction. It will also show with the batch payment transaction in the bank reconciliation Find & Match screen. Depending on your individual bank, the detail may also show on the bank statement you import into Xero.  # noqa: E501

        :param code: The code of this BatchPaymentDetails.  # noqa: E501
        :type: str
        """
        if code is not None and len(code) > 12:
            raise ValueError(
                "Invalid value for `code`, " "length must be less than or equal to `12`"
            )  # noqa: E501

        self._code = code

    @property
    def reference(self):
        """Gets the reference of this BatchPaymentDetails.  # noqa: E501

        (NZ Only) Optional references for the batch payment transaction. It will also show with the batch payment transaction in the bank reconciliation Find & Match screen. Depending on your individual bank, the detail may also show on the bank statement you import into Xero.  # noqa: E501

        :return: The reference of this BatchPaymentDetails.  # noqa: E501
        :rtype: str
        """
        return self._reference

    @reference.setter
    def reference(self, reference):
        """Sets the reference of this BatchPaymentDetails.

        (NZ Only) Optional references for the batch payment transaction. It will also show with the batch payment transaction in the bank reconciliation Find & Match screen. Depending on your individual bank, the detail may also show on the bank statement you import into Xero.  # noqa: E501

        :param reference: The reference of this BatchPaymentDetails.  # noqa: E501
        :type: str
        """
        if reference is not None and len(reference) > 12:
            raise ValueError(
                "Invalid value for `reference`, "
                "length must be less than or equal to `12`"
            )  # noqa: E501

        self._reference = reference



# coding: utf-8

"""
    Xero Payroll UK

    This is the Xero Payroll API for orgs in the UK region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class Problem(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "type": "str",
        "title": "str",
        "status": "str",
        "detail": "str",
        "instance": "str",
        "invalid_fields": "list[InvalidField]",
    }

    attribute_map = {
        "type": "type",
        "title": "title",
        "status": "status",
        "detail": "detail",
        "instance": "instance",
        "invalid_fields": "invalidFields",
    }

    def __init__(
        self,
        type=None,
        title=None,
        status=None,
        detail=None,
        instance=None,
        invalid_fields=None,
    ):  # noqa: E501
        """Problem - a model defined in OpenAPI"""  # noqa: E501

        self._type = None
        self._title = None
        self._status = None
        self._detail = None
        self._instance = None
        self._invalid_fields = None
        self.discriminator = None

        if type is not None:
            self.type = type
        if title is not None:
            self.title = title
        if status is not None:
            self.status = status
        if detail is not None:
            self.detail = detail
        if instance is not None:
            self.instance = instance
        if invalid_fields is not None:
            self.invalid_fields = invalid_fields

    @property
    def type(self):
        """Gets the type of this Problem.  # noqa: E501

        The type of error format  # noqa: E501

        :return: The type of this Problem.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this Problem.

        The type of error format  # noqa: E501

        :param type: The type of this Problem.  # noqa: E501
        :type: str
        """

        self._type = type

    @property
    def title(self):
        """Gets the title of this Problem.  # noqa: E501

        The type of the error  # noqa: E501

        :return: The title of this Problem.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this Problem.

        The type of the error  # noqa: E501

        :param title: The title of this Problem.  # noqa: E501
        :type: str
        """

        self._title = title

    @property
    def status(self):
        """Gets the status of this Problem.  # noqa: E501

        The error status code  # noqa: E501

        :return: The status of this Problem.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this Problem.

        The error status code  # noqa: E501

        :param status: The status of this Problem.  # noqa: E501
        :type: str
        """

        self._status = status

    @property
    def detail(self):
        """Gets the detail of this Problem.  # noqa: E501

        A description of the error  # noqa: E501

        :return: The detail of this Problem.  # noqa: E501
        :rtype: str
        """
        return self._detail

    @detail.setter
    def detail(self, detail):
        """Sets the detail of this Problem.

        A description of the error  # noqa: E501

        :param detail: The detail of this Problem.  # noqa: E501
        :type: str
        """

        self._detail = detail

    @property
    def instance(self):
        """Gets the instance of this Problem.  # noqa: E501


        :return: The instance of this Problem.  # noqa: E501
        :rtype: str
        """
        return self._instance

    @instance.setter
    def instance(self, instance):
        """Sets the instance of this Problem.


        :param instance: The instance of this Problem.  # noqa: E501
        :type: str
        """

        self._instance = instance

    @property
    def invalid_fields(self):
        """Gets the invalid_fields of this Problem.  # noqa: E501


        :return: The invalid_fields of this Problem.  # noqa: E501
        :rtype: list[InvalidField]
        """
        return self._invalid_fields

    @invalid_fields.setter
    def invalid_fields(self, invalid_fields):
        """Sets the invalid_fields of this Problem.


        :param invalid_fields: The invalid_fields of this Problem.  # noqa: E501
        :type: list[InvalidField]
        """

        self._invalid_fields = invalid_fields



# coding: utf-8

"""
    Xero Files API

    These endpoints are specific to Xero Files API  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class Folder(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "name": "str",
        "file_count": "int",
        "email": "str",
        "is_inbox": "bool",
        "id": "str",
    }

    attribute_map = {
        "name": "Name",
        "file_count": "FileCount",
        "email": "Email",
        "is_inbox": "IsInbox",
        "id": "Id",
    }

    def __init__(
        self, name=None, file_count=None, email=None, is_inbox=None, id=None
    ):  # noqa: E501
        """Folder - a model defined in OpenAPI"""  # noqa: E501

        self._name = None
        self._file_count = None
        self._email = None
        self._is_inbox = None
        self._id = None
        self.discriminator = None

        if name is not None:
            self.name = name
        if file_count is not None:
            self.file_count = file_count
        if email is not None:
            self.email = email
        if is_inbox is not None:
            self.is_inbox = is_inbox
        if id is not None:
            self.id = id

    @property
    def name(self):
        """Gets the name of this Folder.  # noqa: E501

        The name of the folder  # noqa: E501

        :return: The name of this Folder.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Folder.

        The name of the folder  # noqa: E501

        :param name: The name of this Folder.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def file_count(self):
        """Gets the file_count of this Folder.  # noqa: E501

        The number of files in the folder  # noqa: E501

        :return: The file_count of this Folder.  # noqa: E501
        :rtype: int
        """
        return self._file_count

    @file_count.setter
    def file_count(self, file_count):
        """Sets the file_count of this Folder.

        The number of files in the folder  # noqa: E501

        :param file_count: The file_count of this Folder.  # noqa: E501
        :type: int
        """

        self._file_count = file_count

    @property
    def email(self):
        """Gets the email of this Folder.  # noqa: E501

        The email address used to email files to the inbox. Only the inbox will have this element.  # noqa: E501

        :return: The email of this Folder.  # noqa: E501
        :rtype: str
        """
        return self._email

    @email.setter
    def email(self, email):
        """Sets the email of this Folder.

        The email address used to email files to the inbox. Only the inbox will have this element.  # noqa: E501

        :param email: The email of this Folder.  # noqa: E501
        :type: str
        """

        self._email = email

    @property
    def is_inbox(self):
        """Gets the is_inbox of this Folder.  # noqa: E501

        to indicate if the folder is the Inbox. The Inbox cannot be renamed or deleted.  # noqa: E501

        :return: The is_inbox of this Folder.  # noqa: E501
        :rtype: bool
        """
        return self._is_inbox

    @is_inbox.setter
    def is_inbox(self, is_inbox):
        """Sets the is_inbox of this Folder.

        to indicate if the folder is the Inbox. The Inbox cannot be renamed or deleted.  # noqa: E501

        :param is_inbox: The is_inbox of this Folder.  # noqa: E501
        :type: bool
        """

        self._is_inbox = is_inbox

    @property
    def id(self):
        """Gets the id of this Folder.  # noqa: E501

        Xero unique identifier for a folder  Files  # noqa: E501

        :return: The id of this Folder.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Folder.

        Xero unique identifier for a folder  Files  # noqa: E501

        :param id: The id of this Folder.  # noqa: E501
        :type: str
        """

        self._id = id



# coding: utf-8

"""
    Xero Payroll AU API

    This is the Xero Payroll API for orgs in Australia region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class LeaveApplication(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "leave_application_id": "str",
        "employee_id": "str",
        "leave_type_id": "str",
        "title": "str",
        "start_date": "date[ms-format]",
        "end_date": "date[ms-format]",
        "description": "str",
        "leave_periods": "list[LeavePeriod]",
        "updated_date_utc": "datetime[ms-format]",
        "validation_errors": "list[ValidationError]",
    }

    attribute_map = {
        "leave_application_id": "LeaveApplicationID",
        "employee_id": "EmployeeID",
        "leave_type_id": "LeaveTypeID",
        "title": "Title",
        "start_date": "StartDate",
        "end_date": "EndDate",
        "description": "Description",
        "leave_periods": "LeavePeriods",
        "updated_date_utc": "UpdatedDateUTC",
        "validation_errors": "ValidationErrors",
    }

    def __init__(
        self,
        leave_application_id=None,
        employee_id=None,
        leave_type_id=None,
        title=None,
        start_date=None,
        end_date=None,
        description=None,
        leave_periods=None,
        updated_date_utc=None,
        validation_errors=None,
    ):  # noqa: E501
        """LeaveApplication - a model defined in OpenAPI"""  # noqa: E501

        self._leave_application_id = None
        self._employee_id = None
        self._leave_type_id = None
        self._title = None
        self._start_date = None
        self._end_date = None
        self._description = None
        self._leave_periods = None
        self._updated_date_utc = None
        self._validation_errors = None
        self.discriminator = None

        if leave_application_id is not None:
            self.leave_application_id = leave_application_id
        if employee_id is not None:
            self.employee_id = employee_id
        if leave_type_id is not None:
            self.leave_type_id = leave_type_id
        if title is not None:
            self.title = title
        if start_date is not None:
            self.start_date = start_date
        if end_date is not None:
            self.end_date = end_date
        if description is not None:
            self.description = description
        if leave_periods is not None:
            self.leave_periods = leave_periods
        if updated_date_utc is not None:
            self.updated_date_utc = updated_date_utc
        if validation_errors is not None:
            self.validation_errors = validation_errors

    @property
    def leave_application_id(self):
        """Gets the leave_application_id of this LeaveApplication.  # noqa: E501

        The Xero identifier for Payroll Employee  # noqa: E501

        :return: The leave_application_id of this LeaveApplication.  # noqa: E501
        :rtype: str
        """
        return self._leave_application_id

    @leave_application_id.setter
    def leave_application_id(self, leave_application_id):
        """Sets the leave_application_id of this LeaveApplication.

        The Xero identifier for Payroll Employee  # noqa: E501

        :param leave_application_id: The leave_application_id of this LeaveApplication.  # noqa: E501
        :type: str
        """

        self._leave_application_id = leave_application_id

    @property
    def employee_id(self):
        """Gets the employee_id of this LeaveApplication.  # noqa: E501

        The Xero identifier for Payroll Employee  # noqa: E501

        :return: The employee_id of this LeaveApplication.  # noqa: E501
        :rtype: str
        """
        return self._employee_id

    @employee_id.setter
    def employee_id(self, employee_id):
        """Sets the employee_id of this LeaveApplication.

        The Xero identifier for Payroll Employee  # noqa: E501

        :param employee_id: The employee_id of this LeaveApplication.  # noqa: E501
        :type: str
        """

        self._employee_id = employee_id

    @property
    def leave_type_id(self):
        """Gets the leave_type_id of this LeaveApplication.  # noqa: E501

        The Xero identifier for Leave Type  # noqa: E501

        :return: The leave_type_id of this LeaveApplication.  # noqa: E501
        :rtype: str
        """
        return self._leave_type_id

    @leave_type_id.setter
    def leave_type_id(self, leave_type_id):
        """Sets the leave_type_id of this LeaveApplication.

        The Xero identifier for Leave Type  # noqa: E501

        :param leave_type_id: The leave_type_id of this LeaveApplication.  # noqa: E501
        :type: str
        """

        self._leave_type_id = leave_type_id

    @property
    def title(self):
        """Gets the title of this LeaveApplication.  # noqa: E501

        The title of the leave  # noqa: E501

        :return: The title of this LeaveApplication.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this LeaveApplication.

        The title of the leave  # noqa: E501

        :param title: The title of this LeaveApplication.  # noqa: E501
        :type: str
        """

        self._title = title

    @property
    def start_date(self):
        """Gets the start_date of this LeaveApplication.  # noqa: E501

        Start date of the leave (YYYY-MM-DD)  # noqa: E501

        :return: The start_date of this LeaveApplication.  # noqa: E501
        :rtype: date
        """
        return self._start_date

    @start_date.setter
    def start_date(self, start_date):
        """Sets the start_date of this LeaveApplication.

        Start date of the leave (YYYY-MM-DD)  # noqa: E501

        :param start_date: The start_date of this LeaveApplication.  # noqa: E501
        :type: date
        """

        self._start_date = start_date

    @property
    def end_date(self):
        """Gets the end_date of this LeaveApplication.  # noqa: E501

        End date of the leave (YYYY-MM-DD)  # noqa: E501

        :return: The end_date of this LeaveApplication.  # noqa: E501
        :rtype: date
        """
        return self._end_date

    @end_date.setter
    def end_date(self, end_date):
        """Sets the end_date of this LeaveApplication.

        End date of the leave (YYYY-MM-DD)  # noqa: E501

        :param end_date: The end_date of this LeaveApplication.  # noqa: E501
        :type: date
        """

        self._end_date = end_date

    @property
    def description(self):
        """Gets the description of this LeaveApplication.  # noqa: E501

        The Description of the Leave  # noqa: E501

        :return: The description of this LeaveApplication.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this LeaveApplication.

        The Description of the Leave  # noqa: E501

        :param description: The description of this LeaveApplication.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def leave_periods(self):
        """Gets the leave_periods of this LeaveApplication.  # noqa: E501


        :return: The leave_periods of this LeaveApplication.  # noqa: E501
        :rtype: list[LeavePeriod]
        """
        return self._leave_periods

    @leave_periods.setter
    def leave_periods(self, leave_periods):
        """Sets the leave_periods of this LeaveApplication.


        :param leave_periods: The leave_periods of this LeaveApplication.  # noqa: E501
        :type: list[LeavePeriod]
        """

        self._leave_periods = leave_periods

    @property
    def updated_date_utc(self):
        """Gets the updated_date_utc of this LeaveApplication.  # noqa: E501

        Last modified timestamp  # noqa: E501

        :return: The updated_date_utc of this LeaveApplication.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_date_utc

    @updated_date_utc.setter
    def updated_date_utc(self, updated_date_utc):
        """Sets the updated_date_utc of this LeaveApplication.

        Last modified timestamp  # noqa: E501

        :param updated_date_utc: The updated_date_utc of this LeaveApplication.  # noqa: E501
        :type: datetime
        """

        self._updated_date_utc = updated_date_utc

    @property
    def validation_errors(self):
        """Gets the validation_errors of this LeaveApplication.  # noqa: E501

        Displays array of validation error messages from the API  # noqa: E501

        :return: The validation_errors of this LeaveApplication.  # noqa: E501
        :rtype: list[ValidationError]
        """
        return self._validation_errors

    @validation_errors.setter
    def validation_errors(self, validation_errors):
        """Sets the validation_errors of this LeaveApplication.

        Displays array of validation error messages from the API  # noqa: E501

        :param validation_errors: The validation_errors of this LeaveApplication.  # noqa: E501
        :type: list[ValidationError]
        """

        self._validation_errors = validation_errors



# coding: utf-8

"""
    Xero Payroll AU API

    This is the Xero Payroll API for orgs in Australia region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class EarningsLine(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "earnings_rate_id": "str",
        "calculation_type": "EarningsRateCalculationType",
        "annual_salary": "float",
        "number_of_units_per_week": "float",
        "rate_per_unit": "float",
        "normal_number_of_units": "float",
        "amount": "float",
        "number_of_units": "float",
        "fixed_amount": "float",
    }

    attribute_map = {
        "earnings_rate_id": "EarningsRateID",
        "calculation_type": "CalculationType",
        "annual_salary": "AnnualSalary",
        "number_of_units_per_week": "NumberOfUnitsPerWeek",
        "rate_per_unit": "RatePerUnit",
        "normal_number_of_units": "NormalNumberOfUnits",
        "amount": "Amount",
        "number_of_units": "NumberOfUnits",
        "fixed_amount": "FixedAmount",
    }

    def __init__(
        self,
        earnings_rate_id=None,
        calculation_type=None,
        annual_salary=None,
        number_of_units_per_week=None,
        rate_per_unit=None,
        normal_number_of_units=None,
        amount=None,
        number_of_units=None,
        fixed_amount=None,
    ):  # noqa: E501
        """EarningsLine - a model defined in OpenAPI"""  # noqa: E501

        self._earnings_rate_id = None
        self._calculation_type = None
        self._annual_salary = None
        self._number_of_units_per_week = None
        self._rate_per_unit = None
        self._normal_number_of_units = None
        self._amount = None
        self._number_of_units = None
        self._fixed_amount = None
        self.discriminator = None

        self.earnings_rate_id = earnings_rate_id
        if calculation_type is not None:
            self.calculation_type = calculation_type
        if annual_salary is not None:
            self.annual_salary = annual_salary
        if number_of_units_per_week is not None:
            self.number_of_units_per_week = number_of_units_per_week
        if rate_per_unit is not None:
            self.rate_per_unit = rate_per_unit
        if normal_number_of_units is not None:
            self.normal_number_of_units = normal_number_of_units
        if amount is not None:
            self.amount = amount
        if number_of_units is not None:
            self.number_of_units = number_of_units
        if fixed_amount is not None:
            self.fixed_amount = fixed_amount

    @property
    def earnings_rate_id(self):
        """Gets the earnings_rate_id of this EarningsLine.  # noqa: E501

        Xero unique id for earnings rate  # noqa: E501

        :return: The earnings_rate_id of this EarningsLine.  # noqa: E501
        :rtype: str
        """
        return self._earnings_rate_id

    @earnings_rate_id.setter
    def earnings_rate_id(self, earnings_rate_id):
        """Sets the earnings_rate_id of this EarningsLine.

        Xero unique id for earnings rate  # noqa: E501

        :param earnings_rate_id: The earnings_rate_id of this EarningsLine.  # noqa: E501
        :type: str
        """
        if earnings_rate_id is None:
            raise ValueError(
                "Invalid value for `earnings_rate_id`, must not be `None`"
            )  # noqa: E501

        self._earnings_rate_id = earnings_rate_id

    @property
    def calculation_type(self):
        """Gets the calculation_type of this EarningsLine.  # noqa: E501


        :return: The calculation_type of this EarningsLine.  # noqa: E501
        :rtype: EarningsRateCalculationType
        """
        return self._calculation_type

    @calculation_type.setter
    def calculation_type(self, calculation_type):
        """Sets the calculation_type of this EarningsLine.


        :param calculation_type: The calculation_type of this EarningsLine.  # noqa: E501
        :type: EarningsRateCalculationType
        """

        self._calculation_type = calculation_type

    @property
    def annual_salary(self):
        """Gets the annual_salary of this EarningsLine.  # noqa: E501

        Annual salary for earnings line  # noqa: E501

        :return: The annual_salary of this EarningsLine.  # noqa: E501
        :rtype: float
        """
        return self._annual_salary

    @annual_salary.setter
    def annual_salary(self, annual_salary):
        """Sets the annual_salary of this EarningsLine.

        Annual salary for earnings line  # noqa: E501

        :param annual_salary: The annual_salary of this EarningsLine.  # noqa: E501
        :type: float
        """

        self._annual_salary = annual_salary

    @property
    def number_of_units_per_week(self):
        """Gets the number_of_units_per_week of this EarningsLine.  # noqa: E501

        number of units for earning line  # noqa: E501

        :return: The number_of_units_per_week of this EarningsLine.  # noqa: E501
        :rtype: float
        """
        return self._number_of_units_per_week

    @number_of_units_per_week.setter
    def number_of_units_per_week(self, number_of_units_per_week):
        """Sets the number_of_units_per_week of this EarningsLine.

        number of units for earning line  # noqa: E501

        :param number_of_units_per_week: The number_of_units_per_week of this EarningsLine.  # noqa: E501
        :type: float
        """

        self._number_of_units_per_week = number_of_units_per_week

    @property
    def rate_per_unit(self):
        """Gets the rate_per_unit of this EarningsLine.  # noqa: E501

        Rate per unit of the EarningsLine.  # noqa: E501

        :return: The rate_per_unit of this EarningsLine.  # noqa: E501
        :rtype: float
        """
        return self._rate_per_unit

    @rate_per_unit.setter
    def rate_per_unit(self, rate_per_unit):
        """Sets the rate_per_unit of this EarningsLine.

        Rate per unit of the EarningsLine.  # noqa: E501

        :param rate_per_unit: The rate_per_unit of this EarningsLine.  # noqa: E501
        :type: float
        """

        self._rate_per_unit = rate_per_unit

    @property
    def normal_number_of_units(self):
        """Gets the normal_number_of_units of this EarningsLine.  # noqa: E501

        Normal number of units for EarningsLine. Applicable when RateType is \"MULTIPLE\"  # noqa: E501

        :return: The normal_number_of_units of this EarningsLine.  # noqa: E501
        :rtype: float
        """
        return self._normal_number_of_units

    @normal_number_of_units.setter
    def normal_number_of_units(self, normal_number_of_units):
        """Sets the normal_number_of_units of this EarningsLine.

        Normal number of units for EarningsLine. Applicable when RateType is \"MULTIPLE\"  # noqa: E501

        :param normal_number_of_units: The normal_number_of_units of this EarningsLine.  # noqa: E501
        :type: float
        """

        self._normal_number_of_units = normal_number_of_units

    @property
    def amount(self):
        """Gets the amount of this EarningsLine.  # noqa: E501

        Earnings rate amount  # noqa: E501

        :return: The amount of this EarningsLine.  # noqa: E501
        :rtype: float
        """
        return self._amount

    @amount.setter
    def amount(self, amount):
        """Sets the amount of this EarningsLine.

        Earnings rate amount  # noqa: E501

        :param amount: The amount of this EarningsLine.  # noqa: E501
        :type: float
        """

        self._amount = amount

    @property
    def number_of_units(self):
        """Gets the number_of_units of this EarningsLine.  # noqa: E501

        Earnings rate number of units.  # noqa: E501

        :return: The number_of_units of this EarningsLine.  # noqa: E501
        :rtype: float
        """
        return self._number_of_units

    @number_of_units.setter
    def number_of_units(self, number_of_units):
        """Sets the number_of_units of this EarningsLine.

        Earnings rate number of units.  # noqa: E501

        :param number_of_units: The number_of_units of this EarningsLine.  # noqa: E501
        :type: float
        """

        self._number_of_units = number_of_units

    @property
    def fixed_amount(self):
        """Gets the fixed_amount of this EarningsLine.  # noqa: E501

        Earnings rate amount. Only applicable if the EarningsRate RateType is Fixed  # noqa: E501

        :return: The fixed_amount of this EarningsLine.  # noqa: E501
        :rtype: float
        """
        return self._fixed_amount

    @fixed_amount.setter
    def fixed_amount(self, fixed_amount):
        """Sets the fixed_amount of this EarningsLine.

        Earnings rate amount. Only applicable if the EarningsRate RateType is Fixed  # noqa: E501

        :param fixed_amount: The fixed_amount of this EarningsLine.  # noqa: E501
        :type: float
        """

        self._fixed_amount = fixed_amount



# coding: utf-8

"""
    Xero Projects API

    This is the Xero Projects API  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class Project(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "project_id": "str",
        "contact_id": "str",
        "name": "str",
        "currency_code": "CurrencyCode",
        "minutes_logged": "int",
        "total_task_amount": "Amount",
        "total_expense_amount": "Amount",
        "estimate_amount": "Amount",
        "minutes_to_be_invoiced": "int",
        "task_amount_to_be_invoiced": "Amount",
        "task_amount_invoiced": "Amount",
        "expense_amount_to_be_invoiced": "Amount",
        "expense_amount_invoiced": "Amount",
        "project_amount_invoiced": "Amount",
        "deposit": "Amount",
        "deposit_applied": "Amount",
        "credit_note_amount": "Amount",
        "deadline_utc": "datetime",
        "total_invoiced": "Amount",
        "total_to_be_invoiced": "Amount",
        "estimate": "Amount",
        "status": "ProjectStatus",
    }

    attribute_map = {
        "project_id": "projectId",
        "contact_id": "contactId",
        "name": "name",
        "currency_code": "currencyCode",
        "minutes_logged": "minutesLogged",
        "total_task_amount": "totalTaskAmount",
        "total_expense_amount": "totalExpenseAmount",
        "estimate_amount": "estimateAmount",
        "minutes_to_be_invoiced": "minutesToBeInvoiced",
        "task_amount_to_be_invoiced": "taskAmountToBeInvoiced",
        "task_amount_invoiced": "taskAmountInvoiced",
        "expense_amount_to_be_invoiced": "expenseAmountToBeInvoiced",
        "expense_amount_invoiced": "expenseAmountInvoiced",
        "project_amount_invoiced": "projectAmountInvoiced",
        "deposit": "deposit",
        "deposit_applied": "depositApplied",
        "credit_note_amount": "creditNoteAmount",
        "deadline_utc": "deadlineUtc",
        "total_invoiced": "totalInvoiced",
        "total_to_be_invoiced": "totalToBeInvoiced",
        "estimate": "estimate",
        "status": "status",
    }

    def __init__(
        self,
        project_id=None,
        contact_id=None,
        name=None,
        currency_code=None,
        minutes_logged=None,
        total_task_amount=None,
        total_expense_amount=None,
        estimate_amount=None,
        minutes_to_be_invoiced=None,
        task_amount_to_be_invoiced=None,
        task_amount_invoiced=None,
        expense_amount_to_be_invoiced=None,
        expense_amount_invoiced=None,
        project_amount_invoiced=None,
        deposit=None,
        deposit_applied=None,
        credit_note_amount=None,
        deadline_utc=None,
        total_invoiced=None,
        total_to_be_invoiced=None,
        estimate=None,
        status=None,
    ):  # noqa: E501
        """Project - a model defined in OpenAPI"""  # noqa: E501

        self._project_id = None
        self._contact_id = None
        self._name = None
        self._currency_code = None
        self._minutes_logged = None
        self._total_task_amount = None
        self._total_expense_amount = None
        self._estimate_amount = None
        self._minutes_to_be_invoiced = None
        self._task_amount_to_be_invoiced = None
        self._task_amount_invoiced = None
        self._expense_amount_to_be_invoiced = None
        self._expense_amount_invoiced = None
        self._project_amount_invoiced = None
        self._deposit = None
        self._deposit_applied = None
        self._credit_note_amount = None
        self._deadline_utc = None
        self._total_invoiced = None
        self._total_to_be_invoiced = None
        self._estimate = None
        self._status = None
        self.discriminator = None

        if project_id is not None:
            self.project_id = project_id
        if contact_id is not None:
            self.contact_id = contact_id
        self.name = name
        if currency_code is not None:
            self.currency_code = currency_code
        if minutes_logged is not None:
            self.minutes_logged = minutes_logged
        if total_task_amount is not None:
            self.total_task_amount = total_task_amount
        if total_expense_amount is not None:
            self.total_expense_amount = total_expense_amount
        if estimate_amount is not None:
            self.estimate_amount = estimate_amount
        if minutes_to_be_invoiced is not None:
            self.minutes_to_be_invoiced = minutes_to_be_invoiced
        if task_amount_to_be_invoiced is not None:
            self.task_amount_to_be_invoiced = task_amount_to_be_invoiced
        if task_amount_invoiced is not None:
            self.task_amount_invoiced = task_amount_invoiced
        if expense_amount_to_be_invoiced is not None:
            self.expense_amount_to_be_invoiced = expense_amount_to_be_invoiced
        if expense_amount_invoiced is not None:
            self.expense_amount_invoiced = expense_amount_invoiced
        if project_amount_invoiced is not None:
            self.project_amount_invoiced = project_amount_invoiced
        if deposit is not None:
            self.deposit = deposit
        if deposit_applied is not None:
            self.deposit_applied = deposit_applied
        if credit_note_amount is not None:
            self.credit_note_amount = credit_note_amount
        if deadline_utc is not None:
            self.deadline_utc = deadline_utc
        if total_invoiced is not None:
            self.total_invoiced = total_invoiced
        if total_to_be_invoiced is not None:
            self.total_to_be_invoiced = total_to_be_invoiced
        if estimate is not None:
            self.estimate = estimate
        if status is not None:
            self.status = status

    @property
    def project_id(self):
        """Gets the project_id of this Project.  # noqa: E501

        Identifier of the project.  # noqa: E501

        :return: The project_id of this Project.  # noqa: E501
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this Project.

        Identifier of the project.  # noqa: E501

        :param project_id: The project_id of this Project.  # noqa: E501
        :type: str
        """

        self._project_id = project_id

    @property
    def contact_id(self):
        """Gets the contact_id of this Project.  # noqa: E501

        Identifier of the contact this project was created for.  # noqa: E501

        :return: The contact_id of this Project.  # noqa: E501
        :rtype: str
        """
        return self._contact_id

    @contact_id.setter
    def contact_id(self, contact_id):
        """Sets the contact_id of this Project.

        Identifier of the contact this project was created for.  # noqa: E501

        :param contact_id: The contact_id of this Project.  # noqa: E501
        :type: str
        """

        self._contact_id = contact_id

    @property
    def name(self):
        """Gets the name of this Project.  # noqa: E501

        Name of the project.  # noqa: E501

        :return: The name of this Project.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Project.

        Name of the project.  # noqa: E501

        :param name: The name of this Project.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError(
                "Invalid value for `name`, must not be `None`"
            )  # noqa: E501

        self._name = name

    @property
    def currency_code(self):
        """Gets the currency_code of this Project.  # noqa: E501


        :return: The currency_code of this Project.  # noqa: E501
        :rtype: CurrencyCode
        """
        return self._currency_code

    @currency_code.setter
    def currency_code(self, currency_code):
        """Sets the currency_code of this Project.


        :param currency_code: The currency_code of this Project.  # noqa: E501
        :type: CurrencyCode
        """

        self._currency_code = currency_code

    @property
    def minutes_logged(self):
        """Gets the minutes_logged of this Project.  # noqa: E501

        A total of minutes logged against all tasks on the Project.  # noqa: E501

        :return: The minutes_logged of this Project.  # noqa: E501
        :rtype: int
        """
        return self._minutes_logged

    @minutes_logged.setter
    def minutes_logged(self, minutes_logged):
        """Sets the minutes_logged of this Project.

        A total of minutes logged against all tasks on the Project.  # noqa: E501

        :param minutes_logged: The minutes_logged of this Project.  # noqa: E501
        :type: int
        """

        self._minutes_logged = minutes_logged

    @property
    def total_task_amount(self):
        """Gets the total_task_amount of this Project.  # noqa: E501


        :return: The total_task_amount of this Project.  # noqa: E501
        :rtype: Amount
        """
        return self._total_task_amount

    @total_task_amount.setter
    def total_task_amount(self, total_task_amount):
        """Sets the total_task_amount of this Project.


        :param total_task_amount: The total_task_amount of this Project.  # noqa: E501
        :type: Amount
        """

        self._total_task_amount = total_task_amount

    @property
    def total_expense_amount(self):
        """Gets the total_expense_amount of this Project.  # noqa: E501


        :return: The total_expense_amount of this Project.  # noqa: E501
        :rtype: Amount
        """
        return self._total_expense_amount

    @total_expense_amount.setter
    def total_expense_amount(self, total_expense_amount):
        """Sets the total_expense_amount of this Project.


        :param total_expense_amount: The total_expense_amount of this Project.  # noqa: E501
        :type: Amount
        """

        self._total_expense_amount = total_expense_amount

    @property
    def estimate_amount(self):
        """Gets the estimate_amount of this Project.  # noqa: E501


        :return: The estimate_amount of this Project.  # noqa: E501
        :rtype: Amount
        """
        return self._estimate_amount

    @estimate_amount.setter
    def estimate_amount(self, estimate_amount):
        """Sets the estimate_amount of this Project.


        :param estimate_amount: The estimate_amount of this Project.  # noqa: E501
        :type: Amount
        """

        self._estimate_amount = estimate_amount

    @property
    def minutes_to_be_invoiced(self):
        """Gets the minutes_to_be_invoiced of this Project.  # noqa: E501

        Minutes which have not been invoiced across all chargeable tasks in the project.  # noqa: E501

        :return: The minutes_to_be_invoiced of this Project.  # noqa: E501
        :rtype: int
        """
        return self._minutes_to_be_invoiced

    @minutes_to_be_invoiced.setter
    def minutes_to_be_invoiced(self, minutes_to_be_invoiced):
        """Sets the minutes_to_be_invoiced of this Project.

        Minutes which have not been invoiced across all chargeable tasks in the project.  # noqa: E501

        :param minutes_to_be_invoiced: The minutes_to_be_invoiced of this Project.  # noqa: E501
        :type: int
        """

        self._minutes_to_be_invoiced = minutes_to_be_invoiced

    @property
    def task_amount_to_be_invoiced(self):
        """Gets the task_amount_to_be_invoiced of this Project.  # noqa: E501


        :return: The task_amount_to_be_invoiced of this Project.  # noqa: E501
        :rtype: Amount
        """
        return self._task_amount_to_be_invoiced

    @task_amount_to_be_invoiced.setter
    def task_amount_to_be_invoiced(self, task_amount_to_be_invoiced):
        """Sets the task_amount_to_be_invoiced of this Project.


        :param task_amount_to_be_invoiced: The task_amount_to_be_invoiced of this Project.  # noqa: E501
        :type: Amount
        """

        self._task_amount_to_be_invoiced = task_amount_to_be_invoiced

    @property
    def task_amount_invoiced(self):
        """Gets the task_amount_invoiced of this Project.  # noqa: E501


        :return: The task_amount_invoiced of this Project.  # noqa: E501
        :rtype: Amount
        """
        return self._task_amount_invoiced

    @task_amount_invoiced.setter
    def task_amount_invoiced(self, task_amount_invoiced):
        """Sets the task_amount_invoiced of this Project.


        :param task_amount_invoiced: The task_amount_invoiced of this Project.  # noqa: E501
        :type: Amount
        """

        self._task_amount_invoiced = task_amount_invoiced

    @property
    def expense_amount_to_be_invoiced(self):
        """Gets the expense_amount_to_be_invoiced of this Project.  # noqa: E501


        :return: The expense_amount_to_be_invoiced of this Project.  # noqa: E501
        :rtype: Amount
        """
        return self._expense_amount_to_be_invoiced

    @expense_amount_to_be_invoiced.setter
    def expense_amount_to_be_invoiced(self, expense_amount_to_be_invoiced):
        """Sets the expense_amount_to_be_invoiced of this Project.


        :param expense_amount_to_be_invoiced: The expense_amount_to_be_invoiced of this Project.  # noqa: E501
        :type: Amount
        """

        self._expense_amount_to_be_invoiced = expense_amount_to_be_invoiced

    @property
    def expense_amount_invoiced(self):
        """Gets the expense_amount_invoiced of this Project.  # noqa: E501


        :return: The expense_amount_invoiced of this Project.  # noqa: E501
        :rtype: Amount
        """
        return self._expense_amount_invoiced

    @expense_amount_invoiced.setter
    def expense_amount_invoiced(self, expense_amount_invoiced):
        """Sets the expense_amount_invoiced of this Project.


        :param expense_amount_invoiced: The expense_amount_invoiced of this Project.  # noqa: E501
        :type: Amount
        """

        self._expense_amount_invoiced = expense_amount_invoiced

    @property
    def project_amount_invoiced(self):
        """Gets the project_amount_invoiced of this Project.  # noqa: E501


        :return: The project_amount_invoiced of this Project.  # noqa: E501
        :rtype: Amount
        """
        return self._project_amount_invoiced

    @project_amount_invoiced.setter
    def project_amount_invoiced(self, project_amount_invoiced):
        """Sets the project_amount_invoiced of this Project.


        :param project_amount_invoiced: The project_amount_invoiced of this Project.  # noqa: E501
        :type: Amount
        """

        self._project_amount_invoiced = project_amount_invoiced

    @property
    def deposit(self):
        """Gets the deposit of this Project.  # noqa: E501


        :return: The deposit of this Project.  # noqa: E501
        :rtype: Amount
        """
        return self._deposit

    @deposit.setter
    def deposit(self, deposit):
        """Sets the deposit of this Project.


        :param deposit: The deposit of this Project.  # noqa: E501
        :type: Amount
        """

        self._deposit = deposit

    @property
    def deposit_applied(self):
        """Gets the deposit_applied of this Project.  # noqa: E501


        :return: The deposit_applied of this Project.  # noqa: E501
        :rtype: Amount
        """
        return self._deposit_applied

    @deposit_applied.setter
    def deposit_applied(self, deposit_applied):
        """Sets the deposit_applied of this Project.


        :param deposit_applied: The deposit_applied of this Project.  # noqa: E501
        :type: Amount
        """

        self._deposit_applied = deposit_applied

    @property
    def credit_note_amount(self):
        """Gets the credit_note_amount of this Project.  # noqa: E501


        :return: The credit_note_amount of this Project.  # noqa: E501
        :rtype: Amount
        """
        return self._credit_note_amount

    @credit_note_amount.setter
    def credit_note_amount(self, credit_note_amount):
        """Sets the credit_note_amount of this Project.


        :param credit_note_amount: The credit_note_amount of this Project.  # noqa: E501
        :type: Amount
        """

        self._credit_note_amount = credit_note_amount

    @property
    def deadline_utc(self):
        """Gets the deadline_utc of this Project.  # noqa: E501

        Deadline for the project. UTC Date Time in ISO-8601 format.  # noqa: E501

        :return: The deadline_utc of this Project.  # noqa: E501
        :rtype: datetime
        """
        return self._deadline_utc

    @deadline_utc.setter
    def deadline_utc(self, deadline_utc):
        """Sets the deadline_utc of this Project.

        Deadline for the project. UTC Date Time in ISO-8601 format.  # noqa: E501

        :param deadline_utc: The deadline_utc of this Project.  # noqa: E501
        :type: datetime
        """

        self._deadline_utc = deadline_utc

    @property
    def total_invoiced(self):
        """Gets the total_invoiced of this Project.  # noqa: E501


        :return: The total_invoiced of this Project.  # noqa: E501
        :rtype: Amount
        """
        return self._total_invoiced

    @total_invoiced.setter
    def total_invoiced(self, total_invoiced):
        """Sets the total_invoiced of this Project.


        :param total_invoiced: The total_invoiced of this Project.  # noqa: E501
        :type: Amount
        """

        self._total_invoiced = total_invoiced

    @property
    def total_to_be_invoiced(self):
        """Gets the total_to_be_invoiced of this Project.  # noqa: E501


        :return: The total_to_be_invoiced of this Project.  # noqa: E501
        :rtype: Amount
        """
        return self._total_to_be_invoiced

    @total_to_be_invoiced.setter
    def total_to_be_invoiced(self, total_to_be_invoiced):
        """Sets the total_to_be_invoiced of this Project.


        :param total_to_be_invoiced: The total_to_be_invoiced of this Project.  # noqa: E501
        :type: Amount
        """

        self._total_to_be_invoiced = total_to_be_invoiced

    @property
    def estimate(self):
        """Gets the estimate of this Project.  # noqa: E501


        :return: The estimate of this Project.  # noqa: E501
        :rtype: Amount
        """
        return self._estimate

    @estimate.setter
    def estimate(self, estimate):
        """Sets the estimate of this Project.


        :param estimate: The estimate of this Project.  # noqa: E501
        :type: Amount
        """

        self._estimate = estimate

    @property
    def status(self):
        """Gets the status of this Project.  # noqa: E501


        :return: The status of this Project.  # noqa: E501
        :rtype: ProjectStatus
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this Project.


        :param status: The status of this Project.  # noqa: E501
        :type: ProjectStatus
        """

        self._status = status



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class ExpenseClaims(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"expense_claims": "list[ExpenseClaim]"}

    attribute_map = {"expense_claims": "ExpenseClaims"}

    def __init__(self, expense_claims=None):  # noqa: E501
        """ExpenseClaims - a model defined in OpenAPI"""  # noqa: E501

        self._expense_claims = None
        self.discriminator = None

        if expense_claims is not None:
            self.expense_claims = expense_claims

    @property
    def expense_claims(self):
        """Gets the expense_claims of this ExpenseClaims.  # noqa: E501


        :return: The expense_claims of this ExpenseClaims.  # noqa: E501
        :rtype: list[ExpenseClaim]
        """
        return self._expense_claims

    @expense_claims.setter
    def expense_claims(self, expense_claims):
        """Sets the expense_claims of this ExpenseClaims.


        :param expense_claims: The expense_claims of this ExpenseClaims.  # noqa: E501
        :type: list[ExpenseClaim]
        """

        self._expense_claims = expense_claims
